@@ -1,4 +1,4 @@
-# $Id: chats.tcl 1024 2007-03-07 15:58:27Z sergei $
+# $Id: chats.tcl 1429 2008-05-18 12:36:02Z sergei $
 
 package require textutil
 
@@ -147,6 +147,11 @@ proc chat::winid {chatid} {
     return .chat_${connid}_$tag
 }
 
+proc chat::winid_to_chatid {winid} {
+    variable chat_id
+    expr {[info exists chat_id($winid)] ? $chat_id($winid) : ""}
+}
+
 proc chat::chatid {connid jid} {
     return [list $connid $jid]
 }
@@ -199,7 +204,6 @@ hook::add rewrite_message_hook [namespace current]::chat::rewrite_message 99
 
 proc chat::process_message_fallback \
      {connid from id type is_subject subject body err thread priority x} {
-    global font
     variable chats
 
     set chatid [chatid $connid $from]
@@ -253,7 +257,7 @@ proc chat::process_message_fallback \
 
     chat::add_message $chatid $from $type $body $x
     if {[llength $x] > 0} {
-        message::show_dialog $connid $from $type $subject $body $thread $priority $x 0
+        message::show_dialog $connid $from $id $type $subject $body $thread $priority $x 0
     }
 }
 
@@ -340,7 +344,6 @@ proc chat::disconnect_groupchats {connid} {
 hook::add predisconnected_hook [namespace current]::chat::disconnect_groupchats 99
 
 proc chat::open_window {chatid type args} {
-    global font font_bold font_italic font_bold_italic
     global chat_width chat_height
     variable chats
     variable chat_id
@@ -411,13 +414,13 @@ proc chat::open_window {chatid type args} {
 	    -helptext [get_user_status_desc $connid $jid]
 	pack $cw.status.icon -side left
 	if {$options(display_status_description)} {
-	    Label $cw.status.desc -text "($statusdesc($status))" -font $font \
+	    Label $cw.status.desc -text "($statusdesc($status))" \
 		-helptext [get_user_status_desc $connid $jid]
 	    pack $cw.status.desc -side left
 	}
     }
     if {[cequal $type chat]} {
-	menubutton $cw.status.mb -text $jid -font $font \
+	menubutton $cw.status.mb -text $jid \
 	    -menu $cw.status.mb.menu
 	create_user_menu $cw.status.mb.menu $chatid
 	pack $cw.status.mb -side left
@@ -428,7 +431,7 @@ proc chat::open_window {chatid type args} {
 	create_conference_menu $cw.status.mb.menu $chatid
 	pack $cw.status.mb -side left
 
-	entry $cw.status.subject -font $font \
+	entry $cw.status.subject \
 	    -xscrollcommand [list [namespace current]::set_subject_tooltip $chatid]
 	pack $cw.status.subject -side left -fill x -expand yes
 
@@ -448,7 +451,7 @@ proc chat::open_window {chatid type args} {
 	}
     }
 
-    PanedWin $cw.pw0 -side right -pad 0 -width 8
+    PanedWin $cw.pw0 -side right -pad 0 -width 4
     pack $cw.pw0 -fill both -expand yes
 
 
@@ -458,13 +461,13 @@ proc chat::open_window {chatid type args} {
     set isw [ScrolledWindow $cw.isw -scrollbar vertical]
     pack $cw.isw -fill both -expand yes -side bottom -in $dow
     textUndoable $cw.input -width $chat_width \
-	-height [option get $cw inputheight Chat] -font $font -wrap word
+	-height [option get $cw inputheight Chat] -wrap word
     $isw setwidget $cw.input
     [winfo parent $dow] configure -height [winfo reqheight $cw.input]
     set chats(inputwin,$chatid) $cw.input
 
     if {[cequal $type groupchat]} {
-	PanedWin $cw.pw -side bottom -pad 2 -width 8
+	PanedWin $cw.pw -side bottom -pad 1 -width 4
 	pack $cw.pw -fill both -expand yes -in $upw
 
 	set cf [PanedWinAdd $cw.pw -weight 1 -minsize 0]
@@ -502,7 +505,7 @@ proc chat::open_window {chatid type args} {
 
     ::richtext::richtext $cf.chat \
         -width $chat_width -height $chat_height \
-        -font $font -wrap word
+        -wrap word
 
     ::plugins::chatlog::config $cf.chat \
 	-theyforeground [query_optiondb $cw theyforeground] \
@@ -515,6 +518,8 @@ proc chat::open_window {chatid type args} {
 
     $csw setwidget $cf.chat
 
+    reverse_scroll $cf.chat
+
     focus $cw.input
 
     bind $cw.input <Shift-Key-Return> { }
@@ -649,8 +654,6 @@ proc chat::trace_room_nickname_change {connid group nick new_nick} {
     set ix [lsearch -exact [::chat::opened] $chatid]
     if {$ix < 0} return
 
-    set new_cid [::chat::chatid $connid $to]
-
     set msg [format [::msgcat::mc \
 	"%s has changed nick to %s."] \
 	$nick $new_nick]
@@ -664,7 +667,7 @@ proc chat::trace_room_nickname_change {connid group nick new_nick} {
 	for the new nick of the room occupant"]
     ::plugins::urls::render_url $cw url $tooltip {} \
 	-title [::msgcat::mc "Open new conversation"] \
-	-command [list [namespace current]::open_window $new_cid chat]
+	-command [list [namespace current]::open_to_user $connid $to]
     $cw insert end \n
     $cw config -state disabled
 }
@@ -860,10 +863,14 @@ proc chat::add_message {chatid from type body x} {
 	    $chatw insert end "\n"
 	}
 
+	# In the last condition we work around the underscrolling if the chat
+	# window wasn't maped yet. It's unlikely that mapped chat window will
+	# be exactly 1 pixel tall.
 	if {$body != "" && !$options(stop_scroll) && \
 		(!$options(smart_scroll) || \
 		     ($options(smart_scroll) && $scroll) || \
-		     [chat::is_our_jid $chatid $from])} {
+		     [chat::is_our_jid $chatid $from] || \
+		     [winfo height $chatw] == 1)} {
 	    after idle [list catch [list $chatw yview moveto 1]]
 	}
 
@@ -905,14 +912,11 @@ proc chat::open_to_user {connid user args} {
 
     set chatid [chatid $connid $jid]
     set cw [winid $chatid]
-    if {[winfo exists $cw]} {
-	if {$::usetabbar} {
-	    .nb raise [ifacetk::nbpage $cw]
-	}
-	focus -force $cw.input
-    } else {
+    if {![winfo exists $cw]} {
 	chat::open_window $chatid chat
     }
+    raise_win $cw
+    focus -force $cw.input
 
     if {![cequal $msg ""]} {
 	debugmsg chat "SEND_MESSAGE ON OPEN:\
@@ -1123,7 +1127,7 @@ proc chat::redraw_roster {group} {
 	    if {$::plugins::nickcolors::options(use_colored_roster_nicks)} {
 		ifacetk::roster::changeforeground $userswin \
 		    [list $connid $jid] $status \
-		    [plugins::nickcolors::get_nick_color $nick]
+		    [plugins::nickcolors::get_color $nick]
 	    } else {
 		ifacetk::roster::changeforeground $userswin \
 		    [list $connid $jid] $status
@@ -1397,6 +1401,8 @@ proc chat::restore_window {cjid type nick connid jid} {
 
     # TODO: Password?
     open_window $chatid $type
+    set cw [winid $chatid]
+    raise_win $cw
 }
 
 #############################################################################
