@@ -1,4 +1,4 @@
-# $Id: disco.tcl 1038 2007-03-10 14:09:40Z sergei $
+# $Id: disco.tcl 1406 2008-05-05 08:19:13Z sergei $
 
 ##############################################################################
 
@@ -71,7 +71,7 @@ proc disco::request_items {jid node args} {
 proc disco::parse_items {connid jid node handler res child} {
     variable disco
 
-    if {![cequal $res OK]} {
+    if {![string equal $res OK]} {
 	if {$handler != ""} {
 	    eval $handler [list ERR $child]
 	}
@@ -169,7 +169,7 @@ proc disco::parse_info {connid jid node handler res child} {
     variable disco
     variable additional_nodes
 
-    if {![cequal $res OK]} {
+    if {![string equal $res OK]} {
 	if {$handler != ""} {
 	    eval $handler [list ERR $child {} {}]
 	}
@@ -211,9 +211,9 @@ proc disco::parse_info {connid jid node handler res child} {
 		}
 	    }
 	    default {
-		if {[jlib::wrapper:getattr $vars1 xmlns] == "jabber:x:data" && \
+		if {[jlib::wrapper:getattr $vars1 xmlns] == $::NS(data) && \
 			[jlib::wrapper:getattr $vars1 type] == "result"} {
-		    set extras [data::parse_xdata_results $children1]
+		    lappend extras [data::parse_xdata_results $children1 -hidden 1]
 		}
 	    }
 	}
@@ -279,16 +279,23 @@ proc disco::info_query_get_handler {connid from lang child} {
     variable node_handlers
     variable supported_features
     variable feature_handlers
+    variable extra_handlers
 
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set node [jlib::wrapper:getattr $vars node]
 
-    if {![cequal $node ""]} {
+    if {![string equal $node ""]} {
 	if {![info exists supported_nodes($node)]} {
-	    return {error cancel not-allowed}
+	    # Probably temporary node
+	    set res {error cancel not-allowed}
+	    hook::run disco_node_reply_hook \
+		      res info $node $connid $from $lang $child
+	    return $res
 	} else {
-	    set restags [$node_handlers($node) info $connid $from $lang $child]
-	    if {[cequal [lindex $restags 0] error]} {
+	    # Permanent node
+	    set restags [eval $node_handlers($node) \
+			      [list info $connid $from $lang $child]]
+	    if {[string equal [lindex $restags 0] error]} {
 		return $restags
 	    } else {
 		set res [jlib::wrapper:createtag query \
@@ -301,14 +308,14 @@ proc disco::info_query_get_handler {connid from lang child} {
 
 	lappend restags [jlib::wrapper:createtag identity \
 			     -vars [list category client \
-					 type pc \
-					 name Tkabber]]
+					 type     pc \
+					 name     Tkabber]]
 
-	foreach ns $::iq::supported_ns {
-	    lappend restags [jlib::wrapper:createtag feature \
-				 -vars [list var $ns]]
+	foreach h $extra_handlers {
+	    lappend restags [eval $h [list $connid $from $lang]]
 	}
-	foreach ns $supported_features {
+
+	foreach ns [lsort [concat $::iq::supported_ns $supported_features]] {
 	    lappend restags [jlib::wrapper:createtag feature \
 				 -vars [list var $ns]]
 	}
@@ -333,12 +340,18 @@ proc disco::items_query_get_handler {connid from lang child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set node [jlib::wrapper:getattr $vars node]
 
-    if {![cequal $node ""]} {
+    if {![string equal $node ""]} {
 	if {![info exists supported_nodes($node)]} {
-	    return {error cancel not-allowed}
+	    # Probably temporary node
+	    set res {error cancel not-allowed}
+	    hook::run disco_node_reply_hook \
+		      res items $node $connid $from $lang $child
+	    return $res
 	} else {
-	    set restags [$node_handlers($node) items $connid $from $lang $child]
-	    if {[cequal [lindex $restags 0] error]} {
+	    # Permanent node
+	    set restags [eval $node_handlers($node) \
+			      [list items $connid $from $lang $child]]
+	    if {[string equal [lindex $restags 0] error]} {
 		return $restags
 	    } else {
 		set res [jlib::wrapper:createtag query \
@@ -353,10 +366,10 @@ proc disco::items_query_get_handler {connid from lang child} {
 
 	foreach node $root_nodes {
 	    set vars [list jid $myjid]
-	    if {![cequal $supported_nodes($node) ""]} {
+	    if {![string equal $supported_nodes($node) ""]} {
 		lappend vars name [::trans::trans $lang $supported_nodes($node)]
 	    }
-	    if {![cequal $node ""]} {
+	    if {![string equal $node ""]} {
 		lappend vars node $node
 	    }
 	    lappend restags [jlib::wrapper:createtag item \
@@ -379,12 +392,26 @@ proc disco::register_feature {feature {handler ""}} {
     variable supported_features
     variable feature_handlers
 
-    lappend supported_features $feature
+    if {[lsearch $supported_features $feature] < 0} {
+	lappend supported_features $feature
+    }
     set feature_handlers($feature) $handler
 }
 
 ###############################################################################
 
+proc disco::unregister_feature {feature} {
+    variable supported_features
+    variable feature_handlers
+
+    if {[set idx [lsearch $supported_features $feature]] >= 0} {
+	set supported_features [lreplace $supported_features $idx $idx]
+	unset feature_handlers($feature)
+    }
+}
+
+###############################################################################
+
 proc disco::register_node {node handler {name ""}} {
     variable root_nodes
 
@@ -404,6 +431,14 @@ proc disco::register_subnode {node handler {name ""}} {
 
 ###############################################################################
 
+proc disco::register_extra {handler} {
+    variable extra_handlers
+
+    lappend extra_handlers $handler
+}
+
+###############################################################################
+
 proc disco::publish_items {jid node action items args} {
 
     set command ""
@@ -473,7 +508,7 @@ proc disco::browser::open_win {jid args} {
 	}
     }
     if {![info exists connid]} {
-	set connid [jlib::route $jid]
+	set connid [lindex [jlib::connections] 0]
     }
 
     if {$jid == ""} {
@@ -490,7 +525,8 @@ proc disco::browser::open_win {jid args} {
     add_win $w -title [::msgcat::mc "Service Discovery"] \
 	-tabtitle [::msgcat::mc "Discovery"] \
 	-raisecmd [list focus $w.tree] \
-	-class JDisco
+	-class JDisco \
+	-raise 1
 
     set config(fill) 	      [option get $w fill          JDisco]
     set config(featurecolor)  [option get $w featurecolor  JDisco]
@@ -606,7 +642,6 @@ proc disco::browser::draw_info \
      {w connid jid node res identities features extras featured_nodes} {
     variable browser
     variable config
-    global font
 
     set tw $browser(tree,$w)
 
@@ -622,7 +657,7 @@ proc disco::browser::draw_info \
     set desc [item_desc $jid $node $name $nitems]
     set icon ""
 
-    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+    add_line $tw $parent_tag $tnode $icon $desc $data \
 	-fill $config(fill)
 
     if {$res != "OK"} {
@@ -633,7 +668,7 @@ proc disco::browser::draw_info \
 		      [error_to_string $identities]]
 	set icon ""
 	
-	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	add_line $tw $parent_tag $tnode $icon $desc $data \
 	    -fill $config(identitycolor)
 
 	remove_old $tw $parent_tag identity [list $tnode]
@@ -656,23 +691,30 @@ proc disco::browser::draw_info \
 	set desc "$name ($category/$type)"
 	set icon [item_icon $category $type]
 	
-	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	add_line $tw $parent_tag $tnode $icon $desc $data \
 	    -fill $config(identitycolor)
     }
 
     set extranodes {}
     
-    foreach extra $extras {
-	lassign $extra var label values
-	set tnode [jid_to_tag "extra $var $jid $node"]
-	lappend extranodes $tnode
-	set data [list extra $var $connid $jid $node]
-	set value [join $values ", "]
-	set desc "$label ($var): $value"
-	set icon ""
+    foreach eform $extras {
+	foreach extra $eform {
+	    lassign $extra var type label values
+	    if {$type == "hidden"} continue
+	    set tnode [jid_to_tag "extra $var $jid $node"]
+	    lappend extranodes $tnode
+	    set data [list extra $var $connid $jid $node]
+	    set value [join $values ", "]
+	    if {$label != ""} {
+		set desc "$label ($var): $value"
+	    } else {
+		set desc "$var: $value"
+	    }
+	    set icon ""
 	
-	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
-	    -fill $config(identitycolor)
+	    add_line $tw $parent_tag $tnode $icon $desc $data \
+		-fill $config(identitycolor)
+	}
     }
 
     set featurenodes {}
@@ -716,7 +758,7 @@ proc disco::browser::draw_info \
 	set icon ""
 
 	if {![$tw exists $tnode]} {
-	    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	    add_line $tw $parent_tag $tnode $icon $desc $data \
 		     -fill $config(fill)
 	}
     }
@@ -746,7 +788,6 @@ hook::add disco_items_hook \
 proc disco::browser::draw_items {w connid jid node res items} {
     variable browser
     variable config
-    global font
 
     set tw $browser(tree,$w)
 
@@ -763,7 +804,7 @@ proc disco::browser::draw_items {w connid jid node res items} {
     set desc [item_desc $jid $node $name $nitems]
     set icon ""
 
-    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+    add_line $tw $parent_tag $tnode $icon $desc $data \
 	-fill $config(fill)
 
     if {$res != "OK"} {
@@ -779,7 +820,7 @@ proc disco::browser::draw_items {w connid jid node res items} {
 				   [error_to_string $items]]
 	    set icon ""
 	
-	    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	    add_line $tw $parent_tag $tnode $icon $desc $data \
 		     -fill $config(fill)
 
 	    remove_old $tw $parent_tag item [list $tnode]
@@ -818,7 +859,7 @@ proc disco::browser::draw_items {w connid jid node res items} {
 
 	lappend itemnodes $tnode
 
-	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	add_line $tw $parent_tag $tnode $icon $desc $data \
 	    -fill $config(fill)
     }
     remove_old $tw $parent_tag item $itemnodes
@@ -832,7 +873,6 @@ proc disco::browser::draw_items {w connid jid node res items} {
 
 proc disco::browser::negotiate_feature {tw connid jid parent type} {
     variable config
-    global font
 
     lassign [negotiate::send_request $connid $jid $type] res opts
 
@@ -846,7 +886,7 @@ proc disco::browser::negotiate_feature {tw connid jid parent type} {
 		      [error_to_string $opts]]
 	set icon ""
 	
-	add_line $tw $parent $node $icon $desc $data -font $font \
+	add_line $tw $parent $node $icon $desc $data \
 	    -fill $config(optioncolor)
 
 	remove_old $tw $parent option [list $node]
@@ -862,7 +902,7 @@ proc disco::browser::negotiate_feature {tw connid jid parent type} {
 	set desc $opt
 	set icon ""
 	
-	add_line $tw $parent $node $icon $desc $data -font $font \
+	add_line $tw $parent $node $icon $desc $data \
 	    -fill $config(optioncolor)
     }
     remove_old $tw $parent option $optnodes
@@ -947,7 +987,7 @@ proc disco::browser::item_desc {jid node name nitems} {
     } else {
 	set sitems ""
     }
-    if {![cequal $name ""]} {
+    if {![string equal $name ""]} {
 	return "$name$snode ($jid)$sitems"
     } else {
 	return "$jid$snode$sitems"
@@ -1149,7 +1189,7 @@ proc disco::browser::browser_action {action bw tnode} {
             reorder_node $tw $tnode $neworder
 
             foreach child [$tw nodes $tnode] {
-                browser::browser_action $action $bw $child
+                browser_action $action $bw $child
             }
 	}
 
@@ -1179,7 +1219,7 @@ proc disco::browser::browser_action {action bw tnode} {
             reorder_node $tw $tnode $neworder
 
             foreach child [$tw nodes $tnode] {
-                browser::browser_action $action $bw $child
+                browser_action $action $bw $child
             }
 	}
 
@@ -1239,13 +1279,13 @@ proc disco::browser::draginitcmd {bw t tnode top} {
 	lassign [get_category_type $t $tnode] category type
 
 	if {![info exists category]} {
-	    # Using parent tag to get conference category
+	    # Using parent tag to get conference category.
 	    # ??? Which else category could be got from parent?
 	    lassign [get_category_type $t [$t parent $tnode]] category type
 
 	    if {![info exists category] || ($category != "conference")} {
-		set conference ""
-		set type ""
+		# For other JIDs use heuristics from roster code.
+		lassign [roster::get_category_and_subtype $connid $jid] category type
 	    }
 	}
 
@@ -1305,6 +1345,11 @@ proc disco::browser::history_add {bw jid} {
 #}
 
 proc disco::browser::register_feature_handler {feature handler args} {
+    eval [list hook::run browser_register_feature_handler_hook \
+	       $feature $handler] $args
+}
+
+proc disco::browser::register_feature_handler1 {feature handler args} {
     variable browser
 
     set node 0
@@ -1324,6 +1369,9 @@ proc disco::browser::register_feature_handler {feature handler args} {
     }
 }
 
+hook::add browser_register_feature_handler_hook \
+	  disco::browser::register_feature_handler1
+
 # Destroy all (global) state assotiated with the given browser window.
 # Intended to be bound to a <Destroy> event handler for browser windows.
 proc disco::browser::destroy_state {bw} {
@@ -1338,9 +1386,3 @@ proc disco::browser::destroy_state {bw} {
     }
 }
 
-hook::add postload_hook \
-    [list browser::register_ns_handler $::NS(disco_info) disco::browser::open_win \
-	 -desc [list * [::msgcat::mc "Discover service"]]]
-hook::add postload_hook \
-    [list browser::register_ns_handler $::NS(disco_items) disco::browser::open_win \
-	 -desc [list * [::msgcat::mc "Discover service"]]]
