@@ -1,4 +1,4 @@
-# $Id: gpgme.tcl 1024 2007-03-07 15:58:27Z sergei $
+# $Id: gpgme.tcl 1408 2008-05-11 11:29:45Z sergei $
 
 namespace eval ::ssj {}
 
@@ -31,7 +31,7 @@ hook::add draw_message_hook ::ssj::draw_encrypted 6
 
 #############################################################################
 
-proc ::ssj::process_x_encrypted {rowvar bodyvar f x connid from type replyP} {
+proc ::ssj::process_x_encrypted {rowvar bodyvar f x connid from id type replyP} {
     upvar 2 $rowvar row
     upvar 2 $bodyvar body
 
@@ -1327,7 +1327,7 @@ hook::add open_message_post_hook ::ssj::message_buttons
 
 #############################################################################
 
-proc ::ssj::process_x_signed {rowvar bodyvar f x connid from type replyP} {
+proc ::ssj::process_x_signed {rowvar bodyvar f x connid from id type replyP} {
     upvar 2 $rowvar row
     upvar 2 $bodyvar body
 
