@@ -1,4 +1,4 @@
-# $Id: iface.tcl 1093 2007-04-09 18:31:05Z sergei $
+# $Id: iface.tcl 1449 2008-06-05 13:48:38Z sergei $
 
 namespace eval ifacetk {
     variable options
@@ -12,9 +12,23 @@ namespace eval ifacetk {
 	     "Use Tabbed Interface (you need to restart)."] \
 	-group IFace -type boolean
 
-    custom::defvar options(tabs_at_bottom) 0 \
-	[::msgcat::mc "Show tabs below the main window."] \
-	-group IFace -type boolean \
+    custom::defvar options(tabs_side) top \
+	[::msgcat::mc "Side where to place tabs in tabbed mode."] \
+	-group IFace -type options \
+	-values [list top    [::msgcat::mc "Top"] \
+		      bottom [::msgcat::mc "Bottom"] \
+		      left   [::msgcat::mc "Left"] \
+		      right  [::msgcat::mc "Right"]] \
+	-command [namespace current]::configure_tabs
+
+    custom::defvar options(tab_minwidth) 90 \
+	[::msgcat::mc "Minimum width of tab buttons in tabbed mode."] \
+	-group IFace -type integer \
+	-command [namespace current]::configure_tabs
+
+    custom::defvar options(tab_maxwidth) 120 \
+	[::msgcat::mc "Maximum width of tab buttons in tabbed mode."] \
+	-group IFace -type integer \
 	-command [namespace current]::configure_tabs
 
     custom::defvar options(show_toolbar) 1 \
@@ -32,11 +46,16 @@ namespace eval ifacetk {
 	-group IFace -type boolean \
 	-command [namespace current]::switch_statusbar
 
-    if {!$::xlfd_fonts} {
-	custom::defvar options(font) $::default_font \
-	    [::msgcat::mc "Font to use in roster, chat windows etc."] \
+    if {!$XLFDFonts} {
+	custom::defvar options(font) $::default_Chat_font \
+	    [::msgcat::mc "Font to use in chat windows."] \
 	    -group IFace -type font \
-	    -command [namespace current]::switch_font
+	    -command [list [namespace current]::switch_font font Chat]
+
+	custom::defvar options(roster_font) $::default_Roster_font \
+	    [::msgcat::mc "Font to use in roster windows."] \
+	    -group IFace -type font \
+	    -command [list [namespace current]::switch_font roster_font Roster]
     }
 
     custom::defvar options(raise_new_tab) 1 \
@@ -71,6 +90,18 @@ namespace eval ifacetk {
 	[::msgcat::mc "Stored main window state (normal or zoomed)"] \
 	-type string -group Hidden
 
+    custom::defvar status_history [list] \
+	    [::msgcat::mc "History of availability status messages"] \
+	    -type list \
+	    -group Hidden
+
+    custom::defvar options(max_status_history) 10 \
+	    [::msgcat::mc "Maximum number of status messages to keep.\
+		    If the history size reaches this threshold,\
+		    the oldest message will be deleted automatically\
+		    when a new one is recorded."] \
+	    -type integer -group IFace
+
     proc isource {file} {
 	set dir [file dirname [info script]]
 	source [file join $dir $file]
@@ -80,26 +111,59 @@ namespace eval ifacetk {
     isource iroster.tcl
     isource systray.tcl
     
-    variable focused ""
     variable after_focused_id ""
 
     variable number_msg
     variable personal_msg
 
-    namespace export add_win tab_set_updated
+    namespace export raise_win add_win tab_set_updated
 }
 
+source [file join [file dirname [info script]] buttonbar.tcl]
+
 proc ifacetk::configure_tabs {args} {
     variable options
     global usetabbar
+    variable tabs
 
     if {$usetabbar} {
-	if {$options(tabs_at_bottom)} {
-	    set side bottom
+	if {[string is integer -strict $options(tab_minwidth)] && \
+		$options(tab_minwidth) >= 0} {
+	    set minwidth $options(tab_minwidth)
 	} else {
-	    set side top
+	    set minwidth 90
 	}
-	.nb configure -side $side
+	if {[string is integer -strict $options(tab_maxwidth)] && \
+		$options(tab_maxwidth) >= 0} {
+	    set maxwidth $options(tab_maxwidth)
+	} else {
+	    set maxwidth 90
+	}
+
+	switch -- $options(tabs_side) {
+	    bottom {
+		set row 2
+		set col 1
+		set orient horizontal
+	    }
+	    left {
+		set row 1
+		set col 0
+		set orient vertical
+	    }
+	    right {
+		set row 1
+		set col 2
+		set orient vertical
+	    }
+	    default {
+		set row 0
+		set col 1
+		set orient horizontal
+	    }
+	}
+	grid .nb -row $row -column $col -sticky nswe -in $tabs
+	.nb configure -minwidth $minwidth -maxwidth $maxwidth -orient $orient
     }
 }
 
@@ -253,8 +317,8 @@ proc ifacetk::load_descmenu {} {
 		       -command {change_password_dialog}] \
 		  [list command [::msgcat::mc "Edit my info..."] {} {} {} \
 		       -command {
-			    if {![lempty [jlib::connections]]} {
-				set connid [jlib::route ""]
+			    if {[llength [jlib::connections]] > 0} {
+				set connid [lindex [jlib::connections] 0]
 				userinfo::open [jlib::connection_bare_jid $connid] \
 				    -editable 1 -connection $connid
 			    }}] \
@@ -294,6 +358,8 @@ proc ifacetk::load_descmenu {} {
 			 -variable [namespace current]::roster::show_only_online] \
 		    [list checkbutton [::msgcat::mc "Show own resources"] {} {} {} \
 			 -variable [namespace current]::roster::options(show_own_resources)] \
+		    [list checkbutton [::msgcat::mc "Use roster filter"] {} {} {} \
+			 -variable [namespace current]::roster::options(use_filter)] \
 		    [list cascad [::msgcat::mc "Export roster..."] \
 			 export_roster export_roster 0 {}] \
 		    [list cascad [::msgcat::mc "Import roster..."] \
@@ -344,8 +410,6 @@ proc ifacetk::load_descmenu {} {
 		  [list command [::msgcat::mc "Show user or service info..."] {} {} {} \
 		       -command {userinfo::show_info_dialog}] \
 		  {separator} \
-		  [list command [::msgcat::mc "Jabber Browser"] {} {} {} \
-		       -command {browser::open}] \
 		  [list command [::msgcat::mc "Service Discovery"] {} {} {} \
 		       -command {disco::browser::open_win ""}] \
 		  {separator} \
@@ -398,16 +462,10 @@ proc ifacetk::load_descmenu {} {
 
 ###############################################################################
 
-proc ifacetk::get_conf {w option} {
-    return [lindex [$w configure $option] 4]
-}
-
-###############################################################################
-
 proc ifacetk::send_announce_message {resource} {
-    if {[lempty [jlib::connections]]} return
+    if {[llength [jlib::connections]] == 0} return
 
-    set server [jlib::connection_server [jlib::route ""]]
+    set server [jlib::connection_server [lindex [jlib::connections] 0]]
 
     if {$resource == "announce/motd/delete"} {
 	message::send_msg "$server/$resource" -type normal
@@ -468,7 +526,12 @@ proc ifacetk::quick_help_window {} {
     $::tk_modify-Z\t[::msgcat::mc {Undo}]
     $::tk_modify-Shift-Z\t[::msgcat::mc {Redo}]
     Alt-PgUp/Down\t[::msgcat::mc {Scroll chat window up/down}]
-    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Correct word}]"
+    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Correct word}]
+
+[::msgcat::mc Systray:]
+    [::msgcat::mc {Left mouse button}]\t[::msgcat::mc {Show main window}]
+    [::msgcat::mc {Middle mouse button}]\t[::msgcat::mc {Hide main window}]
+    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Popup menu}]"
     $t configure -state disabled
 
     $w draw
@@ -497,12 +560,13 @@ proc ifacetk::about_window {} {
     set m [message $frame.msg -text "
 Tkabber $tkabber_version ($toolkit_version)
 
-Copyright \u00a9 2002-2007 [::msgcat::mc {Alexey Shchepin}]
+Copyright \u00a9 2002-2008 [::msgcat::mc {Alexey Shchepin}]
 \n[::msgcat::mc Authors:]
     [::msgcat::mc {Alexey Shchepin}]
     [::msgcat::mc {Marshall T. Rose}]
     [::msgcat::mc {Sergei Golovan}]
     [::msgcat::mc {Michail Litvak}]
+    [::msgcat::mc {Konstantin Khomoutov}]
 "]
     pack $m -side top -anchor w
 
@@ -523,10 +587,10 @@ Copyright \u00a9 2002-2007 [::msgcat::mc {Alexey Shchepin}]
     $w draw
 }
 
-proc ifacetk::switch_font {args} {
+proc ifacetk::switch_font {font class args} {
     variable options
 
-    set opts [lassign $options(font) family size]
+    set opts [lassign $options($font) family size]
     set args [list -family $family -size $size]
     set bold 0
     set italic 0
@@ -566,9 +630,16 @@ proc ifacetk::switch_font {args} {
 	lappend args -overstrike 0
     }
 
-    eval [list font configure font] $args
-    define_fonts
-    roster::redraw_after_idle
+    eval redefine_fonts $class $args
+    if {$class == "Roster"} {
+	roster::redraw_after_idle
+
+	foreach chatid [chat::opened] {
+	    if {[chat::is_groupchat $chatid]} {
+		chat::redraw_roster_after_idle $chatid
+	    }
+	}
+    }
 }
 
 proc ifacetk::show_ssl_info {} {
@@ -683,10 +754,12 @@ proc ifacetk::create_main_window {} {
     global use_tls
     global descmenu
     variable mf
+    variable rw
     variable rosterwidth
     variable ssl_ind
     variable main_window_title
     variable options
+    variable status_history
 
     set main_window_title "Tkabber"
     wm title . $main_window_title
@@ -748,8 +821,12 @@ proc ifacetk::create_main_window {} {
 
     disallow_presence_change ""
 
-    bind [entry .presence.status -textvariable textstatus] \
-	<Return> {set userstatus $userstatus}
+    set w .presence.status
+    ComboBox $w -textvariable textstatus -values $status_history
+    $w bind <Return> {set userstatus $userstatus}
+    trace add variable ::userstatus write \
+	[list [namespace current]::save_availability_status $w]
+    $w bind <Button-3> [list [namespace current]::show_status_context_menu $w]
     if {$usetabbar} {
 	pack .presence.button -side left
 	pack .presence.status -side left -fill x -expand yes
@@ -757,7 +834,7 @@ proc ifacetk::create_main_window {} {
 	pack .presence.button -side top -anchor w
 	pack .presence.status -side top -fill x -expand yes -anchor w
     }
-    pack .presence -side bottom -anchor w -in [$mf getframe] -fill x
+    grid .presence -row 3 -column 1 -sticky nswe -in [$mf getframe]
 
     if {[winfo exists [set m .presence.button.menu]]} {
 	destroy $m
@@ -783,12 +860,18 @@ proc ifacetk::create_main_window {} {
 	set rosterwidth [winfo pixels . 3c]
     }
 
+    grid columnconfigure [$mf getframe] 1 -weight 1
+    grid rowconfigure [$mf getframe] 1 -weight 1
+
     if {$usetabbar} {
-	set pw [PanedWin [$mf getframe].pw -side bottom -pad 2 -width 8]
-	pack $pw -fill both -expand yes
+	set pw [PanedWin [$mf getframe].pw -side bottom -pad 1 -width 4]
+	grid $pw -row 1 -column 1 -sticky nswe
 	set rw [PanedWinAdd $pw -minsize 0 -weight 0]
 	set nw [PanedWinAdd $pw -minsize 32 -weight 1]
 
+	variable tabs
+	set tabs $nw
+
 	roster::create .roster -width $rosterwidth -height 300 \
 	    -popup [namespace current]::roster::popup_menu \
 	    -grouppopup [namespace current]::roster::group_popup_menu \
@@ -796,16 +879,26 @@ proc ifacetk::create_main_window {} {
 	    -doubleclick [namespace current]::roster::jid_doubleclick \
 	    -draginitcmd [namespace current]::roster::draginitcmd \
 	    -dropcmd [namespace current]::roster::dropcmd
-	pack .roster -expand yes -fill both -side left -in $rw
+	pack .roster -expand yes -fill both -side top -in $rw -anchor w
 
-	if {$options(tabs_at_bottom)} {
-	    set side bottom
-	} else {
-	    set side top
-	}
+	grid columnconfigure $nw 1 -weight 1
+	grid rowconfigure $nw 1 -weight 1
 
-	NoteBook .nb -width 400 -side $side
-	pack .nb -side right -in $nw -fill both -expand yes
+	ButtonBar .nb -orient horizontal \
+		      -pady 1 \
+		      -padx 4 \
+		      -pages .pages
+
+	configure_tabs
+
+	frame $nw.fr -relief raised -bd 1
+	grid $nw.fr -row 1 -column 1 -sticky nsew
+
+	grid columnconfigure $nw.fr 0 -weight 1
+	grid rowconfigure $nw.fr 0 -weight 1
+
+	PagesManager .pages -width 400
+	grid .pages -row 0 -column 0 -padx 2m -pady 2m -in $nw.fr -sticky nsew
 	PanedWinConf $pw 0 -width $rosterwidth
 
 	event add <<CollapseRoster>> <Control-Key-r>
@@ -864,9 +957,9 @@ proc ifacetk::create_main_window {} {
 	.nb bindtabs <<ScrollUp>> [list [namespace current]::tab_move .nb -1]
 	.nb bindtabs <<ScrollDown>> [list [namespace current]::tab_move .nb 1]
 
-	DragSite::register .nb.c -draginitcmd [namespace current]::draginitcmd
-	DropSite::register .nb.c -dropovercmd [namespace current]::dropovercmd \
-	    -dropcmd [namespace current]::dropcmd -droptypes {NoteBookPage}
+	#DragSite::register .nb.c -draginitcmd [namespace current]::draginitcmd
+	#DropSite::register .nb.c -dropovercmd [namespace current]::dropovercmd \
+	#    -dropcmd [namespace current]::dropcmd -droptypes {NoteBookPage}
 
 	set geometry [option get . geometry [winfo class .]]
 	if {$geometry == ""} {
@@ -880,7 +973,7 @@ proc ifacetk::create_main_window {} {
 	    -doubleclick [namespace current]::roster::jid_doubleclick \
 	    -draginitcmd [namespace current]::roster::draginitcmd \
 	    -dropcmd [namespace current]::roster::dropcmd
-	pack .roster -expand yes -fill both -side left -in [$mf getframe]
+	grid .roster -row 1 -column 1 -sticky nswe -in [$mf getframe]
 	set geometry [option get . geometry [winfo class .]]
 	if {$geometry == ""} {
 	    set geometry 200x350
@@ -891,11 +984,6 @@ proc ifacetk::create_main_window {} {
 
     wm geometry . $geometry
 
-    if {$usetabbar} {
-	bind . <FocusIn> [list [namespace current]::get_focus .]
-	bind . <FocusOut> [list [namespace current]::loose_focus .]
-    }
-
     define_alert_colors
     update idletasks
 }
@@ -1084,12 +1172,12 @@ proc ifacetk::update_main_window_title {} {
 
 proc ifacetk::chat_window_is_active {chatid} {
     global usetabbar
-    variable focused
 
     set cw [chat::winid $chatid]
     set w [winfo toplevel $cw]
-    if {[cequal $w $focused] && \
-	    (!$usetabbar || ([cequal [.nb raise] [nbpage $cw]]))} {
+    set f [focus]
+    if {($f != "") && ($w == [winfo toplevel $f]) && \
+	    (!$usetabbar || ([.nb raise] == [nbpage $cw]))} {
 	return 1
     } else {
 	return 0
@@ -1098,7 +1186,6 @@ proc ifacetk::chat_window_is_active {chatid} {
 
 proc ifacetk::add_number_of_messages_to_title {chatid from type body extras} {
     global usetabbar
-    variable focused
     variable number_msg
     variable personal_msg
 
@@ -1168,6 +1255,16 @@ hook::add connected_hook \
 hook::add disconnected_hook \
     [namespace current]::ifacetk::set_main_window_title_on_disconnect
 
+proc ifacetk::raise_win {path} {
+    global usetabbar
+
+    if {[winfo exists $path]} {
+	if {$usetabbar} {
+	    .nb raise [nbpage $path]
+	}
+    }
+}
+
 proc ifacetk::add_win {path args} {
     global usetabbar
     variable options
@@ -1177,6 +1274,7 @@ proc ifacetk::add_win {path args} {
     set class ""
     set raisecmd ""
     set type ""
+    set raise 0
 
     foreach {attr val} $args {
 	switch -- $attr {
@@ -1184,6 +1282,7 @@ proc ifacetk::add_win {path args} {
 	    -tabtitle {set tabtitle $val}
 	    -class    {set class    $val}
 	    -raisecmd {set raisecmd $val}
+	    -raise    {set raise    $val}
 	    -type     {set type     $val}
 	    default   {error "Unknown option $attr"}
 	}
@@ -1199,7 +1298,7 @@ proc ifacetk::add_win {path args} {
 	pack $path -expand yes -fill both -in $f
 	#tkwait visibility $path
 	set ::tabcolors($page) ""
-	if {$options(raise_new_tab) || [llength [.nb pages]] == 1} {
+	if {$raise || $options(raise_new_tab) || [llength [.nb pages]] == 1} {
 	    after idle [list catch [list .nb raise $page]]
 	}
     } else {
@@ -1211,11 +1310,12 @@ proc ifacetk::add_win {path args} {
 	if {$geometry != ""} {
 	    wm geometry $path $geometry
 	}
-	bind $path <FocusIn>  [list [namespace current]::get_focus $path]
-	bind $path <FocusOut> [list [namespace current]::loose_focus $path]
     }
 }
 
+bind Chat <FocusIn>  [list [namespace current]::ifacetk::get_focus %W]
+bind Chat <FocusOut> [list [namespace current]::ifacetk::loose_focus %W]
+
 proc ifacetk::tab_raise {path command} {
     tab_set_updated $path
     if {$command != ""} {
@@ -1224,26 +1324,23 @@ proc ifacetk::tab_raise {path command} {
 }
 
 proc ifacetk::get_focus {path} {
-    variable focused
-    variable after_focused_id
-    variable options
-
     if {![winfo exists $path]} return
-    if {[winfo toplevel $path] != $path} return
-    if {![cequal $path .] && ![info exists chat::chat_id($path)]} return
-
-    if {$focused != $path} {
-	if {$after_focused_id != ""} {
-	    after cancel $after_focused_id
-	}
-	set after_focused_id \
-	    [after $options(update_title_delay) [list [namespace current]::set_title $path]]
-	set focused $path
-
-	hook::run got_focus_hook $path
-    }
+    hook::run got_focus_hook $path
 }
 
+proc ifacetk::on_focus_got {path} {
+    variable options
+    variable after_focused_id
+
+    if {$after_focused_id != ""} {
+	after cancel $after_focused_id
+    }
+    set after_focused_id \
+	[after $options(update_title_delay) \
+	       [list [namespace current]::set_title $path]]
+}
+hook::add got_focus_hook [namespace current]::ifacetk::on_focus_got
+
 proc ifacetk::set_title {path} {
     global usetabbar
     variable number_msg
@@ -1267,21 +1364,20 @@ proc ifacetk::set_title {path} {
 }
 
 proc ifacetk::loose_focus {path} {
-    variable focused
+    if {![winfo exists $path]} return
+    hook::run lost_focus_hook $path
+}
+
+proc ifacetk::on_focus_lost {path} {
     variable after_focused_id
 
-    if {![winfo exists $path]} return
-    if {[winfo toplevel $path] != $path} return
-
-    if {($focused == $path) && ($after_focused_id != "")} {
+    if {$after_focused_id != ""} {
 	after cancel $after_focused_id
 	set after_focused_id ""
     }
-    set focused ""
     balloon::destroy
-
-    hook::run lost_focus_hook $path
 }
+hook::add lost_focus_hook [namespace current]::ifacetk::on_focus_lost
 
 proc ifacetk::tab_move {nb shift args} {
     set len [llength [$nb pages]]
@@ -1331,16 +1427,16 @@ proc ifacetk::define_alert_colors {} {
     global alert_colors
 
     if {$usetabbar} {
-	option add *NoteBook.alertColor0 Black widgetDefault
-	option add *NoteBook.alertColor1 DarkBlue widgetDefault
-	option add *NoteBook.alertColor2 Blue widgetDefault
-	option add *NoteBook.alertColor3 Red widgetDefault
+	option add *alertColor0 Black widgetDefault
+	option add *alertColor1 DarkBlue widgetDefault
+	option add *alertColor2 Blue widgetDefault
+	option add *alertColor3 Red widgetDefault
 
 	set alert_colors [list \
-		[option get .nb alertColor0 NoteBook] \
-		[option get .nb alertColor1 NoteBook] \
-		[option get .nb alertColor2 NoteBook] \
-		[option get .nb alertColor3 NoteBook]]
+		[option get .nb alertColor0 ButtonBar] \
+		[option get .nb alertColor1 ButtonBar] \
+		[option get .nb alertColor2 ButtonBar] \
+		[option get .nb alertColor3 ButtonBar]]
     }
 }
 
@@ -1350,7 +1446,6 @@ proc ifacetk::tab_set_updated {path {updated 0} {level ""}} {
     global usetabbar
     global tabcolors
     global alert_lvls alert_colors
-    variable focused
     variable number_msg
     variable personal_msg
 
@@ -1359,7 +1454,8 @@ proc ifacetk::tab_set_updated {path {updated 0} {level ""}} {
     set page [nbpage $path]
     if {[.nb index $page] < 0} {
 	set page $path
-	set path [pack slaves [.nb getframe $page]]
+	if {[.nb index $page] < 0} return
+	set path [nbpath $page]
     }
 
     set st [wm state .]
@@ -1370,8 +1466,7 @@ proc ifacetk::tab_set_updated {path {updated 0} {level ""}} {
 	set backgroundP 0
     }
     if {(!$updated) && $backgroundP} {
-	set updated 1
-	set level message
+	return
     }
 
     if {$updated && \
@@ -1385,7 +1480,7 @@ proc ifacetk::tab_set_updated {path {updated 0} {level ""}} {
 
     if {!$updated || $tabcolors($page) < $lvlnum} {
 	set color [lindex $alert_colors $lvlnum]
-	.nb itemconfigure $page -foreground $color
+	.nb itemconfigure $page -foreground $color -activeforeground $color
 	set tabcolors($page) $lvlnum
 	hook::run tab_set_updated $path $updated $level
     }
@@ -1441,18 +1536,13 @@ proc ifacetk::switch_presencebar {args} {
 
     if {[winfo exists .presence]} {
 	if {$options(show_presencebar)} {
-	    if {$usetabbar} {
-		set before [.mainframe getframe].pw
-	    } else {
-		set before .roster
-	    }
 	    catch {
-		pack .presence -side bottom -anchor w \
-		    -in [.mainframe getframe] -fill x -before $before
+		grid .presence -row 3 -column 1 \
+		    -sticky nswe -in [.mainframe getframe]
 		.presence.status configure -takefocus 1
 	    }
 	} else {
-	    pack forget .presence
+	    grid forget .presence
 	    .presence.status configure -takefocus 0
 	    if {[focus] == ".presence.status"} {
 	    focus [Widget::focusPrev .presence.status]
@@ -1489,6 +1579,45 @@ proc ifacetk::deiconify {} {
 
 hook::add finload_hook [namespace current]::ifacetk::deiconify 100
 
+proc ifacetk::save_availability_status {w args} {
+    variable status_history
+    variable options
+
+    set avstatus [$w cget -text]
+    if {$avstatus == ""} return
+
+    set ix [$w getvalue]
+    if {$ix >= 0} {
+	set status_history [linsert [lreplace $status_history $ix $ix] end $avstatus]
+    } elseif {[llength $status_history] < $options(max_status_history)} {
+	lappend status_history $avstatus
+    } else {
+	set status_history [linsert [lrange $status_history 1 end] end $avstatus]
+    }
+
+    $w configure -values $status_history
+}
+
+proc ifacetk::show_status_context_menu w {
+    set m $w.menu
+
+    if {![winfo exists $m]} {
+	menu $m -tearoff 0
+	$m add command \
+	    -label [::msgcat::mc "Clear history"] \
+	    -command [list [namespace current]::clear_status_history $w]
+    }
+
+    tk_popup $m [winfo pointerx $w] [winfo pointery $w]
+}
+
+proc ifacetk::clear_status_history w {
+    variable status_history
+
+    set status_history [list]
+    $w configure -values $status_history
+}
+
 proc client:errormsg {message} {
     if {[winfo exists .client_error]} {
 	destroy .client_error
@@ -1514,6 +1643,6 @@ if {$tcl_platform(platform) == "unix"
 }
 
 # Import add_win tab_set_updated to the root namespace:
-namespace import ::ifacetk::add_win ::ifacetk::tab_set_updated
+namespace import ::ifacetk::raise_win ::ifacetk::add_win ::ifacetk::tab_set_updated
 
 # vim:ts=8:sw=4:sts=4:noet
