@@ -1,4 +1,4 @@
-# $Id: iroster.tcl 1085 2007-04-08 14:12:50Z sergei $
+# $Id: iroster.tcl 1430 2008-05-18 13:21:52Z sergei $
 
 Tree .faketree
 foreach {k v} [list background       White \
@@ -86,6 +86,14 @@ namespace eval roster {
 	[::msgcat::mc "Add chats group in roster."] \
 	-type boolean -group Roster \
 	-command [namespace current]::redraw_after_idle
+    custom::defvar options(use_filter) 0 \
+	[::msgcat::mc "Use roster filter."] \
+	-type boolean -group Roster \
+	-command [namespace current]::pack_filter_entry
+    custom::defvar options(match_jids) 0 \
+	[::msgcat::mc "Match contact JIDs in addition to nicknames in roster filter."] \
+	-type boolean -group Roster \
+	-command [namespace current]::redraw_after_idle
     custom::defvar options(free_drop) 1 \
 	[::msgcat::mc "Roster item may be dropped not only over group name\
 but also over any item in group."] \
@@ -96,6 +104,10 @@ but also over any item in group."] \
     custom::defvar options(show_conference_user_info) 0 \
 	[::msgcat::mc "Show detailed info on conference room members in roster item tooltips."] \
 	-type boolean -group Roster
+    custom::defvar options(chat_on_doubleclick) 1 \
+	[::msgcat::mc "If set then open chat window/tab when user doubleclicks roster item.\
+		       Otherwise open normal message window."] \
+	-type boolean -group Roster
 
 #   {jid1 {group1 1 group2 0} jid2 {group3 0 group4 0}}
     custom::defvar collapsed_group_list {} \
@@ -105,6 +117,10 @@ but also over any item in group."] \
     custom::defvar show_offline_group_list {} \
 	[::msgcat::mc "Stored show offline roster groups."] \
 	-type string -group Hidden
+    custom::defvar options(filter) "" \
+	[::msgcat::mc "Roster filter."] \
+	-type string -group Hidden \
+	-command [namespace current]::redraw_after_idle
 
     variable menu_item_idx 0
 
@@ -219,6 +235,45 @@ proc roster::process_item {connid jid name groups subsc ask} {
 hook::add roster_item_hook [namespace current]::roster::process_item 90
 hook::add roster_push_hook [namespace current]::roster::process_item 90
 
+proc roster::create_filter_entry {} {
+    entry .roster_filter -textvariable [namespace current]::options(filter) \
+			 -width 1
+    bind .roster_filter <Escape> \
+	 [list set [namespace current]::options(filter) ""]
+    pack_filter_entry
+}
+
+hook::add finload_hook [namespace current]::roster::create_filter_entry
+
+proc roster::pack_filter_entry {args} {
+    global usetabbar
+    variable options
+
+    if {$options(use_filter)} {
+	if {$usetabbar} {
+	    pack .roster_filter -before .roster \
+				-anchor w \
+				-side bottom \
+				-fill x \
+				-expand no \
+				-in $::ifacetk::rw
+	} else {
+	    grid .roster_filter -row 2 \
+				-column 1 \
+				-sticky we \
+				-in [$::ifacetk::mf getframe]
+	}
+	focus .roster_filter
+    } else {
+	if {$usetabbar} {
+	    pack forget .roster_filter
+	} else {
+	    grid forget .roster_filter
+	}
+    }
+    redraw_after_idle
+}
+
 # TODO: get rid of roster::roster
 proc roster::redraw {} {
     upvar #0 roster::aliases aliases
@@ -318,6 +373,16 @@ proc roster::redraw {} {
 	array unset jidsundergroup
 	array unset groupsundergroup
 	foreach jid [::roster::get_jids $connid] {
+	    if {$options(use_filter) && $options(filter) != ""} {
+		if {[string first [string tolower $options(filter)] \
+				  [string tolower [::roster::get_label $connid $jid]]] < 0} {
+		    if {!$options(match_jids) || \
+			    [string first [string tolower $options(filter)] \
+					  [string tolower $jid]] < 0} {
+			continue
+		    }
+		}
+	    }
 	    if {[info exists ignore_jid($jid)]} continue
 	    set jid_groups [::roster::itemconfig $connid $jid -group]
 	    if {![lempty $jid_groups]} {
@@ -379,7 +444,7 @@ proc roster::redraw {} {
 	}
 	if {$options(show_own_resources)} {
 	    set cgroup [list $own_resources_group_name]
-	    set jid [::jlib::connection_bare_jid $connid]
+	    set jid [tolower_node_and_domain [::jlib::connection_bare_jid $connid]]
 	    set jidsingroup($cgroup) [list $jid]
 	    set groups [linsert $groups 0 [list [join $cgroup "\u0000"] $cgroup]]
 	    roster::itemconfig $connid $jid -subsc both
@@ -849,7 +914,7 @@ proc roster::create {w args} {
     set config(availableforeground)    [option get $w availableforeground Roster]
     set config(chatforeground)         [option get $w chatforeground Roster]
 
-    canvas $w.canvas -bg $config(background) \
+    canvas $c -bg $config(background) \
 	-highlightthickness $::tk_highlightthickness \
 	-scrollregion {0 0 0 0} \
 	-width $width -height $height
@@ -863,10 +928,10 @@ proc roster::create {w args} {
     set iroster($w,singleclick) $singleclickproc
     set iroster($w,doubleclick) $doubleclickproc
 
-    bindscroll $w.canvas
+    bindscroll $c
 
     if {[info exists draginitcmd]} {
-	DragSite::register $w.canvas -draginitcmd $draginitcmd
+	DragSite::register $c -draginitcmd $draginitcmd
     }
 
     set args {}
@@ -877,12 +942,11 @@ proc roster::create {w args} {
 	lappend args -dropcmd $dropcmd
     }
     if {![lempty $args]} {
-	eval [list DropSite::register $w.canvas -droptypes {JID}] $args
+	eval [list DropSite::register $c -droptypes {JID}] $args
     }
 }
 
 proc roster::addline {w type text jid group indent {jids {}} {icon ""} {foreground ""}} {
-    global font
     upvar #0 roster::aliases aliases
     variable roster
     variable iroster
@@ -895,7 +959,7 @@ proc roster::addline {w type text jid group indent {jids {}} {icon ""} {foregrou
     set grouptag [jid_to_tag $group]
 
     set ypad 1
-    set linespace [font metric $font -linespace]
+    set linespace [font metric $::RosterFont -linespace]
     set lineheight [expr {$linespace + $ypad}]
 
     set uy $iroster($w,ypos)
@@ -998,11 +1062,11 @@ proc roster::addline {w type text jid group indent {jids {}} {icon ""} {foregrou
 	    set foreground $config(foreground)
 	}
     }
-    $c create text $x $uy -text $text -anchor nw -font $font \
+    $c create text $x $uy -text $text -anchor nw -font $::RosterFont \
 	-fill $foreground -tags [list jid$tag group$grouptag $type text]
 
     set iroster($w,width) [max $iroster($w,width) \
-			      [expr {$x + [font measure $font $text]}]]
+			      [expr {$x + [font measure $::RosterFont $text]}]]
 
 
     $c bind jid$tag <Any-Enter> \
@@ -1122,7 +1186,9 @@ proc roster::jid_doubleclick {id} {
 ###############################################################################
 
 proc roster::doubleclick_fallback {connid jid category subtype} {
-    if {[cequal $chat::options(default_message_type) chat]} {
+    variable options
+
+    if {$options(chat_on_doubleclick)} {
 	chat::open_to_user $connid $jid
     } else {
 	message::send_dialog -to $jid
@@ -1315,8 +1381,11 @@ proc roster::dropcmd {target source X Y op type data} {
 	    set connid $gr
 	    set gr {}
 	}
+    } elseif {![cequal $tags ""]} {
+	lassign [tag_to_jid [crange [lindex $tags 1] 5 end]] connid
+	set gr {}
     } else {
-	set connid [jlib::route ""]
+	set connid [lindex [jlib::connections] 0]
 	set gr {}
     }
     if {$options(nested)} {
@@ -1515,8 +1584,9 @@ proc roster::add_menu_submenu {m prefix suffix jids} {
 	    $m add separator
 	} elseif {![catch { $m1 entrycget $i -menu } menu]} {
 	    set suffix2 [join [lrange [split $menu .] 2 end] .]
-	    set m2 [menu $m.$suffix2 -tearoff 0]
-	    $m add cascad -label $label -menu $m2
+	    set suffix3 [lindex [split $menu .] end]
+	    set m2 [menu $m.$suffix3 -tearoff 0]
+	    $m add cascade -label $label -menu $m2
 	    add_menu_submenu $m2 $prefix .$suffix2 $jids
 	} elseif {![catch { $m1 entrycget $i -variable } var]} {
 	    # Can't distinguish checkbuttons and radiobuttons
@@ -1544,13 +1614,14 @@ proc roster::add_command_submenu {m prefix suffix i label jids} {
     }
     if {[llength $command_list] > 1} {
 	set m2 [menu $m.$i -tearoff 0]
-	$m add cascad -label $label -menu $m2
+	$m add cascade -label $label -menu $m2
 	foreach jid $jids {
 	    set m1 $prefix[jid_to_tag $jid]$suffix
 	    set idx [$m1 index $label]
 	    if {$idx != "none"} {
 		set command [$m1 entrycget $idx -command]
-		$m2 add command -label $jid -command $command
+		$m2 add command -label $jid \
+		    -command [string map [list $m1 $m2] $command]
 	    }
 	}
     } else {
@@ -1575,7 +1646,7 @@ proc roster::add_checkbutton_submenu {m prefix suffix i label jids} {
     }
     if {[llength $command_list] > 1} {
 	set m2 [menu $m.$i -tearoff 0]
-	$m add cascad -label $label -menu $m2
+	$m add cascade -label $label -menu $m2
 	foreach jid $jids {
 	    set m1 $prefix[jid_to_tag $jid]$suffix
 	    set idx [$m1 index $label]
@@ -1698,9 +1769,6 @@ proc roster::create_group_popup_menu {connid name} {
 	set oname $name
     }
     menu $m -tearoff 0
-    $m add command -label [::msgcat::mc "Rename group..."] \
-	-command [list [namespace current]::rename_group_dialog $connid $name] \
-	-state $state
     $m add command \
 	-label [::msgcat::mc "Send message to all users in group..."] \
 	-command [list ::message::send_dialog \
@@ -1714,6 +1782,9 @@ proc roster::create_group_popup_menu {connid name} {
     $m add checkbutton -label [::msgcat::mc "Show offline users"] \
 	-variable [namespace current]::roster(show_offline,[list $connid $oname]) \
 	-command [list [namespace current]::redraw_after_idle]
+    $m add command -label [::msgcat::mc "Rename group..."] \
+	-command [list [namespace current]::rename_group_dialog $connid $name] \
+	-state $state
     $m add command -label [::msgcat::mc "Remove group..."] \
 	-command [list [namespace current]::remove_group_dialog $connid $name] \
 	-state $state
@@ -1842,43 +1913,6 @@ proc roster::add_group_by_jid_regexp_dialog {} {
 
 ###############################################################################
 
-proc roster::setup_import_export_menus {args} {
-    set emenu [.mainframe getmenu export_roster]
-    set imenu [.mainframe getmenu import_roster]
-
-    if {[winfo exists $emenu]} {
-	destroy $emenu
-    }
-    menu $emenu -tearoff 0
-
-    if {[winfo exists $imenu]} {
-	destroy $imenu
-    }
-    menu $imenu -tearoff 0
-
-    if {[jlib::connections] == {}} {
-	.mainframe setmenustate export_roster disabled
-	.mainframe setmenustate import_roster disabled
-    } else {
-	.mainframe setmenustate export_roster normal
-	.mainframe setmenustate import_roster normal
-    }
-
-    foreach c [jlib::connections] {
-	set jid [jlib::connection_jid $c]
-	set label [format [::msgcat::mc "Roster of %s"] $jid]
-	set ecommand [list roster::export_to_file $c]
-	set icommand [list roster::import_from_file $c]
-	$emenu add command -label $label -command $ecommand
-	$imenu add command -label $label -command $icommand
-    }
-}
-hook::add connected_hook [namespace current]::roster::setup_import_export_menus
-hook::add disconnected_hook [namespace current]::roster::setup_import_export_menus
-hook::add finload_hook [namespace current]::roster::setup_import_export_menus
-
-###############################################################################
-
 proc roster::add_group_custom_presence_menu {m connid name} {
     set mm [menu $m.custom_presence -tearoff 0]
 
@@ -1895,7 +1929,7 @@ proc roster::add_group_custom_presence_menu {m connid name} {
     $mm add command -label [::msgcat::mc "Unavailable"] \
 	-command [list roster::send_custom_presence_group $connid $name unavailable]
 
-    $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
+    $m add cascade -label [::msgcat::mc "Send custom presence"] -menu $mm
 }
 
 ###############################################################################
