@@ -1,4 +1,4 @@
-# $Id: systray.tcl 966 2007-02-23 20:08:40Z sergei $
+# $Id: systray.tcl 1282 2007-10-26 17:40:59Z sergei $
 
 # Tray icon support.
 # See also plugins/unix/systray.tcl, plugins/unix/dokingtray.tcl,
@@ -97,8 +97,7 @@ proc systray::destroy {icon} {
 proc systray::popupmenu {m} {
     set tearoff [set [namespace parent]::options(show_tearoffs)]
 
-    menu $m -title [::msgcat::mc "Tkabber Systray"] -tearoff $tearoff \
-	-postcommand [list [namespace code post] $m]
+    menu $m -title [::msgcat::mc "Tkabber Systray"] -tearoff $tearoff
     $m add command -label [::msgcat::mc "About"] \
 	-command [list [namespace parent]::about_window]
     $m add separator
@@ -126,10 +125,10 @@ proc systray::popupmenu {m} {
 	incr id
     }
     $m add separator
-    $m add command -label [::msgcat::mc "Hide Main Window"] \
-		   -command [namespace code withdraw]
-    $m add command -label [::msgcat::mc "Show Main Window"] \
+    $m add command -label [::msgcat::mc "Show main window"] \
 		   -command [namespace code restore]
+    $m add command -label [::msgcat::mc "Hide main window"] \
+		   -command [namespace code withdraw]
     $m add separator
     $m add command -label [::msgcat::mc "Quit"] -command quit
 
@@ -138,32 +137,6 @@ proc systray::popupmenu {m} {
 
 ##########################################################################
 
-proc systray::post {m} {
-    global curuserstatus
-
-    switch -- [wm state .] {
-        withdrawn {
-            set state1 normal
-            set state2 disabled
-        }
-        default {
-            set state1 disabled
-            set state2 normal
-        }
-    }
-    if {$curuserstatus == "unavailable"} {
-	set state0 disabled
-    } else {
-	set state0 normal
-    }
-
-    $m entryconfigure [$m index [::msgcat::mc "Presence"]] -state $state0
-    $m entryconfigure [$m index [::msgcat::mc "Show Main Window"]] -state $state1
-    $m entryconfigure [$m index [::msgcat::mc "Hide Main Window"]] -state $state2
-}
-
-##########################################################################
-
 # Withdraws the main Tkabber window from the screen:
 proc systray::withdraw {} {
     variable saved_state
@@ -213,7 +186,11 @@ proc systray::restore {} {
 	withdrawn {
 	    reshow
 	}
+	default {
+	    wm deiconify .
+	}
     }
+    raise .
 }
 
 proc systray::toggle_state {} {
@@ -297,7 +274,7 @@ proc systray::toggle {icon ff} {
 
     set hitP 0
     foreach {k v} [array get tabcolors] {
-	if {![winfo exists .nb.f$k]} {
+	if {[.nb index $k] < 0} {
 	    continue
 	}
 	if {(![cequal $v ""]) && ($v > $hitP)} {
