@@ -1,4 +1,4 @@
-# $Id: login.tcl 1038 2007-03-10 14:09:40Z sergei $
+# $Id: login.tcl 1442 2008-05-21 11:36:30Z sergei $
 
 
 if {[lcontain [jlib::capabilities transport] tls]} {
@@ -19,7 +19,7 @@ if {[lcontain [jlib::capabilities auth] sasl]} {
     set have_sasl 0
 }
 
-if {[lcontain [jlib::capabilities proxy] http]} {
+if {[llength [jlib::capabilities proxy]] > 1} {
     set have_proxy 1
 } else {
     set have_proxy 0
@@ -68,6 +68,11 @@ custom::defvar loginconf(allowauthplain) 0 \
 		   is transmitted unencrypted)."] \
 	-group Login -type boolean
 
+custom::defvar loginconf(allowgoogletoken) 1 \
+    [::msgcat::mc "Allow X-GOOGLE-TOKEN authentication mechanisms. It requires\
+		   connection to Google via HTTPS."] \
+	-group Login -type boolean
+
 if {$have_sasl} {
     custom::defvar loginconf(usesasl) 1 \
 	[::msgcat::mc "Use SASL authentication."] \
@@ -103,25 +108,36 @@ if {$use_tls} {
 }
 
 if {$have_proxy} {
-    custom::defvar loginconf(useproxy) 0 \
-	[::msgcat::mc "Use HTTP proxy to connect."] \
-	-group Login -type boolean
-    custom::defvar loginconf(httpproxy) "localhost" \
+    set values {}
+    foreach type [jlib::capabilities proxy] {
+	switch -- $type {
+	    none   {lappend values none   [::msgcat::mc "None"]}
+	    socks4 {lappend values socks4 [::msgcat::mc "SOCKS4a"]}
+	    socks5 {lappend values socks5 [::msgcat::mc "SOCKS5"]}
+	    https  {lappend values https  [::msgcat::mc "HTTPS"]}
+	}
+    }
+    custom::defvar loginconf(proxy) none \
+	[::msgcat::mc "Proxy type to connect."] \
+	-group Login -type options \
+	-values $values
+    custom::defvar loginconf(proxyhost) "localhost" \
 	[::msgcat::mc "HTTP proxy address."] \
 	-group Login -type string
-    custom::defvar loginconf(httpproxyport) 3128 \
+    custom::defvar loginconf(proxyport) 3128 \
 	[::msgcat::mc "HTTP proxy port."] \
 	-group Login -type integer
-    custom::defvar loginconf(httplogin) "" \
+    custom::defvar loginconf(proxyusername) "" \
 	[::msgcat::mc "HTTP proxy username."] \
 	-group Login -type string
-    custom::defvar loginconf(httppassword) "" \
+    custom::defvar loginconf(proxypassword) "" \
 	[::msgcat::mc "HTTP proxy password."] \
 	-group Login -type password
-    custom::defvar loginconf(httpuseragent) \
+    custom::defvar loginconf(proxyuseragent) \
 	"Mozilla/4.0 (compatible; MSIE 6.0;\
-$::tcl_platform(os) $::tcl_platform(osVersion))" \
-	[::msgcat::mc "User-Agent string."] -group Login -type string
+	 $::tcl_platform(os) $::tcl_platform(osVersion))" \
+	[::msgcat::mc "User-Agent string."] \
+	-group Login -type string
 }
 
 custom::defvar loginconf(usealtserver) 0 \
@@ -156,10 +172,10 @@ before creating new key sequence."] \
 	[::msgcat::mc "Timeout for waiting for HTTP poll responses (if set\
 to zero, Tkabber will wait forever)."] \
 	-group Login -type integer
-    custom::defvar loginconf(pollmin) 3000 \
+    custom::defvar loginconf(pollmin) 6000 \
 	[::msgcat::mc "Minimum poll interval."] \
 	-group Login -type integer
-    custom::defvar loginconf(pollmax) 30000 \
+    custom::defvar loginconf(pollmax) 60000 \
 	[::msgcat::mc "Maximum poll interval."] \
 	-group Login -type integer
 }
@@ -167,8 +183,39 @@ to zero, Tkabber will wait forever)."] \
 custom::defvar reasonlist {} [::msgcat::mc "List of logout reasons."] \
 	-group Hidden
 
+######################################################################
+
+# connect errors mapping
+
+array set connect_error [list \
+	err_unknown 			[::msgcat::mc "Unknown error"] \
+	timeout				[::msgcat::mc "Timeout"] \
+	network-failure			[::msgcat::mc "Network failure"] \
+	err_authorization_required	[::msgcat::mc "Proxy authentication required"] \
+	err_version			[::msgcat::mc "Incorrect SOCKS version"] \
+	err_unsupported_method		[::msgcat::mc "Unsupported SOCKS method"] \
+	err_authentication_unsupported	[::msgcat::mc "Unsupported SOCKS authentication method"] \
+	err_authorization		[::msgcat::mc "SOCKS authentication failed"] \
+	rsp_failure			[::msgcat::mc "SOCKS request failed"] \
+	rsp_errconnect			[::msgcat::mc "SOCKS server cannot identify username"] \
+	rsp_erruserid			[::msgcat::mc "SOCKS server username identification failed"] \
+	rsp_notallowed			[::msgcat::mc "SOCKS connection not allowed by ruleset"] \
+	rsp_netunreachable		[::msgcat::mc "Network unreachable"] \
+	rsp_hostunreachable		[::msgcat::mc "Host unreachable"] \
+	rsp_refused			[::msgcat::mc "Connection refused by destination host"] \
+	rsp_expired			[::msgcat::mc "TTL expired"] \
+	rsp_cmdunsupported		[::msgcat::mc "SOCKS command not supported"] \
+	rsp_addrunsupported		[::msgcat::mc "Address type not supported by SOCKS proxy"] \
+	err_unknown_address_type	[::msgcat::mc "Unknown address type"]]
+
+# TLS info
+#
+# [::msgcat::mc "Certificate has expired"]
+# [::msgcat::mc "Self signed certificate"]
+
 ######################################################################
 proc login {logindata} {
+    global connect_error
     global login_after_time
     global login_after_id
 
@@ -188,8 +235,13 @@ proc login {logindata} {
 	    if {[winfo exists .connect_err]} {
 		destroy .connect_err
 	    }
+	    if {[info exists connect_error($connid)]} {
+		set msg $connect_error($connid)
+	    } else {
+		set msg $connid
+	    }
 	    set res [MessageDlg .connect_err -width 600 -icon error \
-		-message [format [::msgcat::mc "Failed to connect: %s"] $connid] \
+		-message [format [::msgcat::mc "Failed to connect: %s"] $msg] \
 		-type user -buttons [list abort [::msgcat::mc "Keep trying"]] \
 		-default 0 -cancel 0]
 	    if {$res} {
@@ -246,8 +298,18 @@ proc client:tls_callback {connid args} {
 	    }
 	    append info [::msgcat::mc ". Proceed?\n\n"]
 	    foreach {k v} [lindex $args 3] {
-		if {![cequal $v ""] && [info exists ssl_certificate_fields($k)]} {
-		    append info [format "%s: %s\n" $ssl_certificate_fields($k) $v]
+		switch -- $k {
+		    subject - issuer {
+			set v [regsub -all {\s*[/,]\s*(\w+=)} $v \n\t\\1]
+		    }
+		}
+		if {![cequal $v ""]} {
+		    if {[info exists ssl_certificate_fields($k)]} {
+			append info [format "%s: %s\n" \
+					    $ssl_certificate_fields($k) $v]
+		    } else {
+			append info [format "%s: %s\n" $k $v]
+		    }
 		}
 	    }
 
@@ -294,18 +356,19 @@ proc login_connect {logindata} {
     set tls_warning_info($connid) ""
 
     set args [list -password $lc(password) \
-		   -allowauthplain $lc(allowauthplain)]
+		   -allowauthplain $lc(allowauthplain) \
+		   -allowgoogletoken $lc(allowgoogletoken)]
     if {$have_sasl} {
 	lappend args -usesasl $lc(usesasl)
     }
 
-    if {$have_proxy && $lc(useproxy)} {
-	lappend args -proxytype http
-	lappend args -proxyhost $lc(httpproxy)
-	lappend args -proxyport $lc(httpproxyport)
-	lappend args -proxyusername $lc(httplogin)
-	lappend args -proxypassword $lc(httppassword)
-	lappend args -proxyuseragent $lc(httpuseragent)
+    if {$have_proxy && ($lc(proxy) != "none")} {
+	lappend args -proxy $lc(proxy)
+	lappend args -proxyhost $lc(proxyhost)
+	lappend args -proxyport $lc(proxyport)
+	lappend args -proxyusername $lc(proxyusername)
+	lappend args -proxypassword $lc(proxypassword)
+	lappend args -proxyuseragent $lc(proxyuseragent)
     }
 
     set ascii_server [idna::domain_toascii $lc(server)]
@@ -322,9 +385,9 @@ proc login_connect {logindata} {
 				-pollint $lc(pollmin) \
 				-pollmin $lc(pollmin) \
 				-pollmax $lc(pollmax) \
-				-pollurl $url \
-				-pollusekeys $lc(usepollkeys) \
-				-pollnumkeys $lc(numberofpollkeys)] $args]
+				-httpurl $url \
+				-httpusekeys $lc(usepollkeys) \
+				-httpnumkeys $lc(numberofpollkeys)] $args]
     } else {
 	if {$have_compress && $lc(stream_options) == "compressed"} {
 	    lappend args -usecompression 1
@@ -473,12 +536,12 @@ proc recv_auth_result {connid logindata res args} {
     if {$res == "OK"} {
 	connected $connid $logindata
     } else {
-	if {[winfo exists .auth_err]} {
-	    destroy .auth_err
+	if {[winfo exists .auth_err$connid]} {
+	    destroy .auth_err$connid
 	}
 	lassign [error_type_condition [lindex $args 0]] type cond
 	if {($type == "sasl") || ($type == "auth" && $cond == "not-authorized")} {
-	    set res [MessageDlg .auth_err -aspect 50000 -icon error \
+	    set res [MessageDlg .auth_err$connid -aspect 50000 -icon error \
 			 -message [format \
 			     [::msgcat::mc "Authentication failed: %s\nCreate new account?"] \
 			     [error_to_string [lindex $args 0]]] \
@@ -496,7 +559,7 @@ proc recv_auth_result {connid logindata res args} {
 		return
 	    }
 	} else {
-	    MessageDlg .auth_err -aspect 50000 -icon error \
+	    MessageDlg .auth_err$connid -aspect 50000 -icon error \
 		-message [format \
 			     [::msgcat::mc "Authentication failed: %s"] \
 			     [error_to_string [lindex $args 0]]] \
@@ -511,10 +574,10 @@ proc recv_register_result {connid logindata res args} {
 	jlib::disconnect
 	login $logindata
     } else {
-	if {[winfo exists .auth_err]} {
-	    destroy .auth_err
+	if {[winfo exists .auth_err$connid]} {
+	    destroy .auth_err$connid
 	}
-	MessageDlg .auth_err -aspect 50000 -icon error \
+	MessageDlg .auth_err$connid -aspect 50000 -icon error \
 	    -message [format [::msgcat::mc "Registration failed: %s"] \
 			  [error_to_string [lindex $args 0]]] \
 	    -type user -buttons ok -default 0 -cancel 0
@@ -635,31 +698,31 @@ proc show_logout_dialog {} {
 
     set lw .logout
 
-    if {![winfo exists $lw]} {
-        Dialog $lw -title [::msgcat::mc "Logout with reason"] \
+    if {[winfo exists $lw]} {
+	destroy $lw
+    }
+
+    Dialog $lw -title [::msgcat::mc "Logout with reason"] \
 	    -separator 1 -anchor e -default 0 -cancel 1
 
-        set lf [$lw getframe]
-        grid columnconfigure $lf 1 -weight 1
+    set lf [$lw getframe]
+    grid columnconfigure $lf 1 -weight 1
 
-	if {[llength $reasonlist]} {set reason [lindex $reasonlist 0]}
+    if {[llength $reasonlist]} {set reason [lindex $reasonlist 0]}
 
-        label $lf.lreason   -text    [::msgcat::mc "Reason:"]
-        ecursor_entry [ComboBox $lf.reason -textvariable reason \
-		-values $reasonlist -width 35].e
-        label $lf.lpriority -text    [::msgcat::mc "Priority:"]
-        ecursor_entry [entry $lf.priority -textvariable loginconf(priority)]
+    label $lf.lreason   -text    [::msgcat::mc "Reason:"]
+    ecursor_entry [ComboBox $lf.reason -textvariable reason \
+	    -values $reasonlist -width 35].e
+    label $lf.lpriority -text    [::msgcat::mc "Priority:"]
+    ecursor_entry [entry $lf.priority -textvariable loginconf(priority)]
 
-        grid $lf.lreason   -row 0 -column 0 -sticky e
-        grid $lf.reason    -row 0 -column 1 -sticky ew
-        grid $lf.lpriority -row 1 -column 0 -sticky e
-        grid $lf.priority  -row 1 -column 1 -sticky ew
+    grid $lf.lreason   -row 0 -column 0 -sticky e
+    grid $lf.reason    -row 0 -column 1 -sticky ew
+    grid $lf.lpriority -row 1 -column 0 -sticky e
+    grid $lf.priority  -row 1 -column 1 -sticky ew
 
-        $lw add -text [::msgcat::mc "Log out"] -command logout_reason
-        $lw add -text [::msgcat::mc "Cancel"] -command "$lw withdraw"
-    } else {
-        set lf [$lw getframe]
-    }
+    $lw add -text [::msgcat::mc "Log out"] -command logout_reason
+    $lw add -text [::msgcat::mc "Cancel"] -command "$lw withdraw"
 
     $lw draw $lf.reason
 }
@@ -670,7 +733,9 @@ proc logout_reason {} {
     set reasonlist [update_combo_list $reasonlist $reason 10]
 
     set lw .logout
-    $lw withdraw
+    if {[winfo exists $lw]} {
+	destroy $lw
+    }
 
     # TODO
     set logoutpriority $::loginconf(priority)
@@ -678,7 +743,6 @@ proc logout_reason {} {
     set logoutuserstatus unavailable
 
     logout
-
-    destroy $lw
 }
 
+# vim:ts=8:sw=4:sts=4:noet
