@@ -1,4 +1,4 @@
-# $Id: muc.tcl 1099 2007-04-12 06:44:00Z sergei $
+# $Id: muc.tcl 1408 2008-05-11 11:29:45Z sergei $
 
 # Multi-User Chat support (XEP-0045)
 
@@ -12,7 +12,11 @@ namespace eval muc {
 	-type boolean -group Chat
     custom::defvar options(gen_muc_status_change_msgs) 0 \
 	[::msgcat::mc "Generate groupchat messages when occupant\
-	    changes his/her status and/or status message"] \
+	    changes his/her status and/or status message."] \
+	-type boolean -group Chat
+    custom::defvar options(gen_muc_position_change_msgs) 0 \
+	[::msgcat::mc "Generate groupchat messages when occupant's\
+	    room position (affiliation and/or role) changes."] \
 	-type boolean -group Chat
     custom::defvar options(propose_configure) 0 \
 	[::msgcat::mc "Propose to configure newly created MUC room.\
@@ -38,6 +42,18 @@ namespace eval muc {
 	-type boolean -group IQ
 }
 
+# MUC affiliations (for translation):
+# [::msgcat::mc "owner"]
+# [::msgcat::mc "admin"]
+# [::msgcat::mc "member"]
+# [::msgcat::mc "outcast"]
+# [::msgcat::mc "none"]
+# MUC roles (for translation):
+# [::msgcat::mc "moderator"]
+# [::msgcat::mc "participant"]
+# [::msgcat::mc "visitor"]
+# (yet another "none" omitted)
+
 ###############################################################################
 
 set ::NS(muc)       http://jabber.org/protocol/muc
@@ -174,43 +190,35 @@ proc muc::handle_commands {chatid user body type} {
     } elseif {[cequal [crange $body 0 6] "/voice "]} {
 	set params {role participant}
 	set dir up
-	set nick [parse_nick $body 7]
-	set reason ""
+	lassign [parse_nick_reason $body 7] nick reason
     } elseif {[cequal [crange $body 0 8] "/devoice "]} {
 	set params {role visitor}
 	set dir down
-	set nick [parse_nick $body 9]
-	set reason ""
+	lassign [parse_nick_reason $body 9] nick reason
     } elseif {[cequal [crange $body 0 7] "/member "]} {
 	set params {affiliation member}
 	set dir up
-	set nick [parse_nick $body 8]
-	set reason ""
+	lassign [parse_nick_reason $body 8] nick reason
     } elseif {[cequal [crange $body 0 9] "/demember "]} {
 	set params {affiliation none}
 	set dir down
-	set nick [parse_nick $body 10]
-	set reason ""
+	lassign [parse_nick_reason $body 10] nick reason
     } elseif {[cequal [crange $body 0 10] "/moderator "]} {
 	set params {role moderator}
 	set dir up
-	set nick [parse_nick $body 11]
-	set reason ""
+	lassign [parse_nick_reason $body 11] nick reason
     } elseif {[cequal [crange $body 0 12] "/demoderator "]} {
 	set params {role participant}
 	set dir down
-	set nick [parse_nick $body 13]
-	set reason ""
+	lassign [parse_nick_reason $body 13] nick reason
     } elseif {[cequal [crange $body 0 6] "/admin "]} {
 	set params {affiliation admin}
 	set dir up
-	set nick [parse_nick $body 7]
-	set reason ""
+	lassign [parse_nick_reason $body 7] nick reason
     } elseif {[cequal [crange $body 0 8] "/deadmin "]} {
 	set params {affiliation member}
 	set dir down
-	set nick [parse_nick $body 9]
-	set reason ""
+	lassign [parse_nick_reason $body 9] nick reason
     } else {
 	return
     }
@@ -549,9 +557,35 @@ proc muc::receive_list {attr val chatid res child} {
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
     bind $w <Destroy> [list after idle [list muc::list_cleanup $w $f]]
 
-    button $w.add -text [::msgcat::mc "Add"] \
+    frame $w.fr
+    pack $w.fr -side bottom -in $wf -fill x
+
+    button $w.add -text [::msgcat::mc "Add new item"] \
 	-command [list muc::list_add_item $sf $f $attr $val]
-    pack $w.add -side bottom -anchor e -in $wf -padx 1m -pady 1m
+    pack $w.add -side right -in $w.fr -padx 1m -pady 1m
+
+    label $w.lall -text [::msgcat::mc "All items:"]
+    pack $w.lall -side left -in $w.fr -padx 1m -pady 1m
+
+    switch -- $attr {
+	role {
+	    ComboBox $w.roleall -text $val \
+		-values {moderator participant visitor none} \
+		-editable no \
+		-width 11 \
+		-modifycmd [list muc::change_all_items $f $w.roleall $attr]
+	    pack $w.roleall -side left -anchor w -in $w.fr -pady 1m
+	}
+	affiliation {
+	    ComboBox $w.affiliationall -text $val \
+		-values {owner admin member none outcast} \
+		-editable no \
+		-width 7 \
+		-modifycmd [list muc::change_all_items $f $w.affiliationall $attr]
+	    pack $w.affiliationall -side left -in $w.fr -pady 1m
+	}
+    }
+
     pack $sw -side top -expand yes -fill both
 
     bindscroll $f $sf
@@ -576,8 +610,20 @@ proc muc::receive_list {attr val chatid res child} {
 
 ###############################################################################
 
+proc muc::change_all_items {f combobox attr} {
+    variable listdata
+    variable origlistdata
+
+    set value [$combobox get]
+
+    for {set i 1} {$i <= $listdata($f,rows)} {incr i} {
+	set listdata($f,$attr,$i) $value
+    }
+}
+
+###############################################################################
+
 proc muc::fill_list {sf f items attr val} {
-    global font
     variable listdata
     variable origlistdata
 
@@ -635,7 +681,7 @@ proc muc::fill_list {sf f items attr val} {
 	lassign $item listdata($f,nick,$row) listdata($f,jid,$row) \
 		      role affiliation listdata($f,reason,$row)
 
-	entry $f.nick$row -font $font \
+	entry $f.nick$row \
 	    -textvariable muc::listdata($f,nick,$row) \
 	    -takefocus 0 \
 	    -highlightthickness 0 \
@@ -646,7 +692,7 @@ proc muc::fill_list {sf f items attr val} {
 	grid $f.nick$row -row $row -column 0 -sticky we -padx 1m
 	bindscroll $f.nick$row $sf
 
-	entry $f.jid$row -font $font \
+	entry $f.jid$row \
 	    -textvariable muc::listdata($f,jid,$row) \
 	    -takefocus 0 \
 	    -highlightthickness 0 \
@@ -678,7 +724,7 @@ proc muc::fill_list {sf f items attr val} {
 	    }
 	}
 
-	entry $f.reason$row -font $font \
+	entry $f.reason$row \
 	    -textvariable muc::listdata($f,reason,$row) \
 	    -width 40
 	grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
@@ -694,20 +740,17 @@ proc muc::fill_list {sf f items attr val} {
 ###############################################################################
 
 proc muc::list_add_item {sf f attr val} {
-    global font
     variable listdata
 
     set row [incr listdata($f,rows)]
 
     entry $f.nick$row \
-	-font $font \
 	-textvariable muc::listdata($f,nick,$row) \
 	-width 20
     grid $f.nick$row -row $row -column 0 -sticky we -padx 1m
     bindscroll $f.nick$row $sf
     
     entry $f.jid$row \
-	-font $font \
 	-textvariable muc::listdata($f,jid,$row) \
 	-width 30
     grid $f.jid$row -row $row -column 1 -sticky we -padx 1m
@@ -735,7 +778,6 @@ proc muc::list_add_item {sf f attr val} {
     }
 
     entry $f.reason$row \
-	-font $font \
 	-textvariable muc::listdata($f,reason,$row) \
 	-width 40
     grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
@@ -972,8 +1014,8 @@ proc muc::process_muc_user {connid user type childrens} {
 	    item {
 		if {$type != "unavailable"} {
 		    set users(jid,$connid,$user) [jlib::wrapper:getattr $vars jid]
-		    set users(role,$connid,$user) [jlib::wrapper:getattr $vars role]
-		    set users(affiliation,$connid,$user) \
+		    track_room_position $connid $user \
+			[jlib::wrapper:getattr $vars role] \
 			[jlib::wrapper:getattr $vars affiliation]
 		} else {
 		    set new_nick [jlib::wrapper:getattr $vars nick]
@@ -1016,6 +1058,9 @@ proc muc::process_muc_user {connid user type childrens} {
 		set chatid [chat::chatid $connid $group]
 		switch -- $code {
 		    201 {
+			chat::add_message \
+			    $chatid $group groupchat \
+			    [::msgcat::mc "A new room is created"] {}
 			# 201: room creation
 			if {$options(propose_configure)} {
 			    request_config_dialog $chatid
@@ -1139,6 +1184,7 @@ proc muc::process_unavailable {chatid nick} {
 
 proc muc::report_available {chatid nick entered} {
     variable options
+    variable users
     variable ignore_available
 
     set connid [::chat::get_connid $chatid]
@@ -1161,6 +1207,11 @@ proc muc::report_available {chatid nick entered} {
 	    set occupant $nick
 	}
 	set msg [format [::msgcat::mc "%s has entered"] $occupant]
+	if {$options(gen_muc_position_change_msgs)} {
+	    append msg " " [::msgcat::mc "as %s/%s" \
+		[::msgcat::mc $users(affiliation,$connid,$jid)] \
+		[::msgcat::mc $users(role,$connid,$jid)]]
+	}
     }
     catch { unset ignore_available }
 
@@ -1181,6 +1232,50 @@ proc muc::report_available {chatid nick entered} {
     ::chat::add_message $chatid $group groupchat $msg {}
 }
 
+proc muc::track_room_position {connid jid role affiliation} {
+    variable options
+    variable users
+
+    upvar 0 users(role,$connid,$jid) _role \
+	users(affiliation,$connid,$jid) _aff
+
+    set group  [node_and_server_from_jid $jid]
+    if {![is_compatible $group]} return
+
+    set chatid [chat::chatid $connid $group]
+
+    if {[chat::is_opened $chatid] && $options(gen_muc_position_change_msgs)} {
+	set nick [chat::get_nick $connid $jid groupchat]
+
+	set role_changed [expr {![info exists _role] \
+	    || ![string equal $role $_role]}]
+	set aff_changed  [expr {![info exists _aff] \
+	    || ![string equal $affiliation $_aff]}]
+
+	if {$aff_changed} {
+	    if {$role_changed} {
+		set msg [::msgcat::mc "%s has been assigned a new room position: %s/%s" \
+		    $nick [::msgcat::mc $affiliation] [::msgcat::mc $role]]
+	    } else {
+		set msg [::msgcat::mc "%s has been assigned a new affiliation: %s" \
+		    $nick [::msgcat::mc $affiliation]]
+	    }
+	} elseif {$role_changed} {
+	    set msg [::msgcat::mc "%s has been assigned a new role: %s" \
+		$nick [::msgcat::mc $role]]
+	} else {
+	    set msg ""
+	}
+
+	if {$msg != ""} {
+	    ::chat::add_message $chatid $group groupchat $msg {}
+	}
+    }
+
+    set _aff  $affiliation
+    set _role $role
+}
+
 ###############################################################################
 
 proc muc::change_nick {chatid nick} {
@@ -1331,10 +1426,16 @@ proc muc::join_group {connid group nick {password ""}} {
     variable muc_password
 
     set group [tolower_node_and_domain $group]
+    set chatid [chat::chatid $connid $group]
+
+    if {[info exists chat::chats(status,$chatid)] && \
+	    ![string equal $chat::chats(status,$chatid) disconnected]} {
+	change_nick $chatid $nick
+	return
+    }
 
     privacy::add_to_special_list $connid conference [server_from_jid $group]
 
-    set chatid [chat::chatid $connid $group]
     set_our_groupchat_nick $chatid $nick
 
     chat::open_window $chatid groupchat
@@ -1429,7 +1530,7 @@ proc muc::invite_xconference {connid group jid reason} {
 
 ###############################################################################
 
-proc muc::process_invitation {rowvar bodyvar f x connid from type replyP} {
+proc muc::process_invitation {rowvar bodyvar f x connid from id type replyP} {
     upvar 2 $rowvar row
     upvar 2 $bodyvar body
 
@@ -1557,21 +1658,12 @@ proc muc::join {jid args} {
     }
 }
 
-hook::add postload_hook \
-    [list browser::register_ns_handler jabber:iq:conference muc::join \
-	 -desc [list conference [::msgcat::mc "Join conference"]]]
 hook::add postload_hook \
     [list disco::browser::register_feature_handler jabber:iq:conference \
 	 muc::join -desc [list conference [::msgcat::mc "Join conference"]]]
-hook::add postload_hook \
-    [list browser::register_ns_handler $::NS(muc) muc::join \
-	 -desc [list conference [::msgcat::mc "Join conference"]]]
 hook::add postload_hook \
     [list disco::browser::register_feature_handler $::NS(muc) muc::join \
 	 -desc [list conference [::msgcat::mc "Join conference"]]]
-hook::add postload_hook \
-    [list browser::register_ns_handler "gc-1.0" muc::join \
-	 -desc [list conference [::msgcat::mc "Join groupchat"]]]
 hook::add postload_hook \
     [list disco::browser::register_feature_handler "gc-1.0" muc::join \
 	 -desc [list conference [::msgcat::mc "Join groupchat"]]]
