@@ -1,13 +1,12 @@
-# $Id: draw_timestamp.tcl 924 2007-02-11 12:40:21Z sergei $
+# $Id: draw_timestamp.tcl 1442 2008-05-21 11:36:30Z sergei $
 
 custom::defvar options(timestamp_format) {[%R]} \
     [::msgcat::mc "Format of timestamp in chat message.\
-Refer to Tcl documentation of 'clock' command for description of format.
-
-Examples:
-  \[%R\] \u2013 \[20:37\]
-  \[%T\] \u2013 \[20:37:12\]
-  \[%a %b %d %H:%M:%S %Z %Y\] \u2013 \[Thu Jan 01 03:00:00 MSK 1970\]"] \
+		   Refer to Tcl documentation of 'clock' command for\
+		   description of format.\n\nExamples:\n \
+		   \[%R\] - \[20:37\]\n  \[%T\] - \[20:37:12\]\n \
+		   \[%a %b %d %H:%M:%S %Z %Y\] -\
+		   \[Thu Jan 01 03:00:00 MSK 1970\]"] \
     -type string -group Chat
 custom::defvar options(delayed_timestamp_format) {[%m/%d %R]} \
     [::msgcat::mc "Format of timestamp in delayed chat messages delayed\
@@ -20,7 +19,7 @@ proc draw_timestamp {chatid from type body x} {
     set chatw [chat::chat_win $chatid]
 
     set seconds [jlib::x_delay $x]
-    set seconds1 [clock scan "-23 hours 59 minutes" -base [clock seconds]]
+    set seconds1 [expr {[clock seconds] - 24*60*60 + 60}]
     if {$seconds <= $seconds1} {
 	set format $options(delayed_timestamp_format)
     } else {
