@@ -1,4 +1,4 @@
-# $Id: headlines.tcl 1042 2007-03-10 16:18:58Z sergei $
+# $Id: headlines.tcl 1429 2008-05-18 12:36:02Z sergei $
 
 #############################################################################
 
@@ -82,7 +82,6 @@ proc headlines::get_tree {connid from} {
 
 proc headlines::open_window {connid from} {
     global tcl_platform
-    global font
     variable options
     variable trees
 
@@ -114,9 +113,9 @@ proc headlines::open_window {connid from} {
 
     add_win $hw -title $title -tabtitle $tabtitle \
 		-raisecmd [list focus $tw] \
-		-class JBrowser
+		-class JDisco
 
-    PanedWin $hw.pw -side right -pad 0 -width 8
+    PanedWin $hw.pw -side right -pad 0 -width 4
     pack $hw.pw -fill both -expand yes
 
     set uw [PanedWinAdd $hw.pw -weight 1]
@@ -124,7 +123,7 @@ proc headlines::open_window {connid from} {
 
     frame $dw.date
     label $dw.date.label -anchor w -text [::msgcat::mc "Date:"]
-    entry $dw.date.ts -font $font \
+    entry $dw.date.ts \
 		      -takefocus 0 \
 		      -highlightthickness 0 \
 		      -relief flat
@@ -134,7 +133,7 @@ proc headlines::open_window {connid from} {
 
     frame $dw.from
     label $dw.from.label -anchor w -text [::msgcat::mc "From:"]
-    entry $dw.from.jid -font $font \
+    entry $dw.from.jid \
 		       -takefocus 0 \
 		       -highlightthickness 0 \
 		       -relief flat
@@ -144,7 +143,7 @@ proc headlines::open_window {connid from} {
 
     frame $dw.subject
     label $dw.subject.lsubj -anchor w -text [::msgcat::mc "Subject:"]
-    text $dw.subject.subj -font $font \
+    text $dw.subject.subj \
 			  -height 1 \
 			  -takefocus 0 \
 			  -highlightthickness 0 \
@@ -162,14 +161,15 @@ proc headlines::open_window {connid from} {
     }
 
     if {![info exists options(seencolor)]} {
-	if {[cequal $tcl_platform(platform) unix]} {
-	    set options(seencolor) [option get $hw disabledForeground JBrowser]
+	if {[string equal $tcl_platform(platform) unix] && \
+		![string equal [option get $hw disabledForeground JDisco] ""]} {
+	    set options(seencolor) [option get $hw disabledForeground JDisco]
 	} else {
-	    set options(seencolor) [option get $hw nscolor JBrowser]
+	    set options(seencolor) [option get $hw featurecolor JDisco]
 	}
     }
     if {![info exists options(unseencolor)]} {
-	set options(unseencolor) [option get $hw fill JBrowser]
+	set options(unseencolor) [option get $hw fill JDisco]
     }
 
     set sw [ScrolledWindow $uw.sw]
@@ -193,7 +193,7 @@ proc headlines::open_window {connid from} {
     bindscroll $tw.c
 
     set dsw [ScrolledWindow $dw.sw]
-    text $hw.body -font $font -height 12 -state disabled \
+    text $hw.body -height 12 -state disabled \
 		  -wrap word -takefocus 1
     ::richtext::config $hw.body -using url
     $dsw setwidget $hw.body
@@ -218,7 +218,6 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     variable headlines
     variable trees
     variable options
-    global font
 
     set subject [string trim $subject]
     set body [string trim $body]
@@ -244,7 +243,13 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
 	}
     }
 
-    if {[cequal $subject ""]} {
+    if {[string equal $subject ""] && [string equal $body ""] && \
+	    [string equal $desc ""] && [string equal $url ""]} {
+	# Ignore an empty message
+	return
+    }
+
+    if {[string equal $subject ""]} {
 	set subject $desc
     } else {
 	if {$options(display_subject_only)} {
@@ -280,19 +285,19 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     set fnode [str2node $text]
     if {![$tw exists $fnode]} {
         $tw insert end root $fnode -text [string map [list "\n" " "] $text] -open 1 \
-            -image browser/headline -font $font \
+            -image browser/headline \
             -fill $options(seencolor) \
             -data [list type from text $text unseen 0]
     }
 
-    if {($options(multiple) > 1) || ([cequal $subject $desc])} {
+    if {($options(multiple) > 1) || ([string equal $subject $desc])} {
         set snode $fnode
     } else {
         set snode $fnode-subject-[str2node $dsubject]
 
         if {![$tw exists $snode]} {
             $tw insert end $fnode $snode -text [string map [list "\n" " "] $dsubject] -open 1 \
-                -image browser/headline -font $font \
+                -image browser/headline \
                 -fill $options(seencolor) \
                 -data [list type subject text $subject unseen 0]
         }
@@ -305,12 +310,13 @@ proc headlines::show {connid from type subject body thread priority x {data {}}}
     array set props [list type article unseen 1 seconds $seconds]
     array set props $data
     array set props [list text $desc url $url body $body]
+
     set nodetext \
-	[cconcat \
-	     [clock format $props(seconds) -format $options(timestamp_format)] \
-		    [string map [list "\n" " "] $ddesc]]
+	     [clock format $props(seconds) -format $options(timestamp_format)]
+    append nodetext [string map [list "\n" " "] $ddesc]
+
     $tw insert end $snode $anode -text $nodetext -open 1 \
-        -fill $options(seencolor) -font $font \
+        -fill $options(seencolor) \
         -data [array get props]
     if {$props(unseen)} {
         $tw itemconfigure $anode -fill $options(unseencolor)
@@ -659,7 +665,7 @@ proc headlines::update {tw node} {
     variable options
 
     for {set parent [$tw parent $node]} \
-            {![cequal $parent root]} \
+            {![string equal $parent root]} \
             {set parent [$tw parent $parent]} {
         set unseen 0
 
@@ -708,7 +714,7 @@ proc headlines::balloon {hw node} {
 
     switch -- $props(type) {
         article {
-            if {![cequal $props(body) ""]} {
+            if {![string equal $props(body) ""]} {
                 return [list $hw:$node $props(body) -width $width]
             }
         }
@@ -878,7 +884,7 @@ proc headlines::forward {menu tw node} {
     set balloons {}
     foreach c [jlib::connections] {
 	foreach choice [roster::get_jids $c] {
-	    if {![cequal [roster::itemconfig $c $choice -category] conference]} {
+	    if {![string equal [roster::itemconfig $c $choice -category] conference]} {
 		lappend choices [list $c $choice] [roster::get_label $c $choice]
 		lappend balloons [list $c $choice] $choice
 	    }
