@@ -20,6 +20,7 @@ if {[lempty [package versions tkcon]] && \
 }
 
 namespace eval tkcon {
+    variable onceP 1
     variable showP 0
 }
 
@@ -36,6 +37,7 @@ proc tkcon::add_tkcon_to_tkabber_menu {args} {
 }
 
 proc tkcon::show_console {} {
+    variable onceP
     variable showP
 
     if {[llength [info commands ::tkcon::*]] <= 0} {
@@ -44,10 +46,23 @@ proc tkcon::show_console {} {
 
     if {$showP} {
 	tkcon show
+
+	if {$onceP} {
+	    wm protocol $::tkcon::PRIV(root) WM_DELETE_WINDOW \
+		    [namespace current]::hide_console
+	    set onceP 0
+	}
     } else {
 	tkcon hide
     }
 }
 
+proc tkcon::hide_console {} {
+    variable showP
+
+    tkcon hide
+    set showP 0
+}
+
 hook::add finload_hook [namespace current]::tkcon::add_tkcon_to_tkabber_menu
 
