@@ -1,4 +1,4 @@
-# $Id: conferences.tcl 1082 2007-04-02 08:12:07Z sergei $
+# $Id: conferences.tcl 1389 2008-03-07 10:28:05Z sergei $
 #
 # Bookmarks (XEP-0048) support (conference bookmarks in roster)
 #
@@ -32,10 +32,8 @@ namespace eval conferences {
     # variable to store roster conference bookmarks
     array set bookmarks {}
 
-    variable NS
-    set NS(private) "jabber:iq:private"
-    set NS(bookmarks) "storage:bookmarks"
-    set NS(tkabber:groups) "tkabber:bookmarks:groups"
+    set ::NS(bookmarks) "storage:bookmarks"
+    set ::NS(tkabber:groups) "tkabber:bookmarks:groups"
 }
 
 ###############################################################################
@@ -57,25 +55,19 @@ hook::add disconnected_hook [namespace current]::conferences::free_bookmarks
 #
 
 proc conferences::request_bookmarks {connid} {
-    variable NS
     variable bookmarks
     variable responds
 
     set responds($connid) 0
     array unset bookmarks $connid,*
 
-    jlib::send_iq get \
-	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $NS(private)] \
-	     -subtags [list [jlib::wrapper:createtag storage \
-				 -vars [list xmlns $NS(bookmarks)]]]] \
+    private::retrieve [list [jlib::wrapper:createtag storage \
+				 -vars [list xmlns $::NS(bookmarks)]]] \
 	-command [list [namespace current]::process_bookmarks $connid] \
 	-connection $connid
-    jlib::send_iq get \
-	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $NS(private)] \
-	     -subtags [list [jlib::wrapper:createtag storage \
-				 -vars [list xmlns $NS(tkabber:groups)]]]] \
+
+    private::retrieve [list [jlib::wrapper:createtag storage \
+				 -vars [list xmlns $::NS(tkabber:groups)]]] \
 	-command [list [namespace current]::process_bookmarks $connid] \
 	-connection $connid
 }
@@ -83,73 +75,130 @@ proc conferences::request_bookmarks {connid} {
 hook::add connected_hook [namespace current]::conferences::request_bookmarks 20
 
 proc conferences::process_bookmarks {connid res child} {
-    variable NS
     variable bookmarks
     variable responds
+    global NS
 
     if {$res != "OK"} return
 
     incr responds($connid)
 
-    jlib::wrapper:splitxml $child tag vars isempty cdata children
-
-    foreach ch $children {
+    foreach ch $child {
 	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 cdata1 children1
 
-	if {[jlib::wrapper:getattr $vars1 xmlns] == $NS(bookmarks)} {
-	    foreach bookmark $children1 {
-		jlib::wrapper:splitxml $bookmark btag bvars bisempty bcdata bchildren
-
-		if {$btag != "conference"} continue
-
-		set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
-		set bookmarks($connid,jid,$jid) $jid
-
-		set bookmarks($connid,name,$jid) [jlib::wrapper:getattr $bvars name]
-		set bookmarks($connid,nick,$jid) ""
-		set bookmarks($connid,password,$jid) ""
-		if {![info exists bookmarks($connid,groups,$jid)]} {
-		    set bookmarks($connid,groups,$jid) {}
+	switch -- [jlib::wrapper:getattr $vars1 xmlns] \
+	    $NS(bookmarks) {
+		foreach bookmark $children1 {
+		    create_muc_bookmark $connid $bookmark
 		}
-
-		set autojoin [jlib::wrapper:getattr $bvars autojoin]
-		switch -- $autojoin {
-		    1 -
-		    true { set bookmarks($connid,autojoin,$jid) 1 }
-		    default { set bookmarks($connid,autojoin,$jid) 0 }
-		}
-		
-		foreach bch $bchildren {
-		    jlib::wrapper:splitxml \
-			$bch tag2 vars2 isempty2 cdata2 children2
-		    switch -- $tag2 {
-			nick { set bookmarks($connid,nick,$jid) $cdata2 }
-			password { set bookmarks($connid,password,$jid) $cdata2 }
-		    }
+	    } \
+	    $NS(tkabber:groups) {
+		foreach bookmark $children1 {
+		    create_muc_bmgroup $connid $bookmark
 		}
 	    }
-	} elseif {[jlib::wrapper:getattr $vars1 xmlns] == $NS(tkabber:groups)} {
-	    foreach bookmark $children1 {
-		jlib::wrapper:splitxml $bookmark btag bvars bisempty bcdata bchildren
+    }
 
-		if {$btag != "conference"} continue
+    if {$responds($connid) < 2} return
 
-		set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
+    push_bookmarks_to_roster $connid
+    after idle [list [namespace current]::autojoin_groups $connid]
+}
 
-		set groups {}
-		foreach bch $bchildren {
-		    jlib::wrapper:splitxml \
-			$bch tag2 vars2 isempty2 cdata2 children2
-		    switch -- $tag2 {
-			group { lappend groups $cdata2 }
-		    }
-		}
-		set bookmarks($connid,groups,$jid) $groups
+proc conferences::create_muc_bookmark {connid xmldata args} {
+    variable bookmarks
+
+    set merge 0
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -merge { set merge $val }
+	    default {
+		return -code error "Bad option \"$opt\":\
+		    must be -merge"
 	    }
 	}
     }
 
-    if {$responds($connid) < 2} return
+    jlib::wrapper:splitxml $xmldata btag bvars bisempty bcdata bchildren
+
+    if {![string equal $btag conference]} { return 0 }
+
+    set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
+
+    if {$merge && [info exists bookmarks($connid,jid,$jid)]} {
+	return 0
+    } else {
+	set bookmarks($connid,jid,$jid) $jid
+
+	set bookmarks($connid,name,$jid) [jlib::wrapper:getattr $bvars name]
+	set bookmarks($connid,nick,$jid) ""
+	set bookmarks($connid,password,$jid) ""
+	if {![info exists bookmarks($connid,groups,$jid)]} {
+	    set bookmarks($connid,groups,$jid) {}
+	    set bookmarks($connid,hasgroups,$jid) 0
+	} else {
+	    set bookmarks($connid,hasgroups,$jid) 1
+	}
+
+	set autojoin [jlib::wrapper:getattr $bvars autojoin]
+	switch -- $autojoin {
+	    1 -
+	    true { set bookmarks($connid,autojoin,$jid) 1 }
+	    default { set bookmarks($connid,autojoin,$jid) 0 }
+	}
+	
+	foreach bch $bchildren {
+	    jlib::wrapper:splitxml \
+		$bch tag2 vars2 isempty2 cdata2 children2
+	    switch -- $tag2 {
+		nick { set bookmarks($connid,nick,$jid) $cdata2 }
+		password { set bookmarks($connid,password,$jid) $cdata2 }
+	    }
+	}
+	return 1
+    }
+}
+
+proc conferences::create_muc_bmgroup {connid xmldata args} {
+    variable bookmarks
+
+    set merge 0
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -merge { set merge $val }
+	    default {
+		return -code error "Bad option \"$opt\":\
+		    must be -merge"
+	    }
+	}
+    }
+
+    jlib::wrapper:splitxml $xmldata btag bvars bisempty bcdata bchildren
+
+    if {![string equal $btag conference]} return
+
+    set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
+
+    set groups [list]
+    foreach bch $bchildren {
+	jlib::wrapper:splitxml $bch tag2 vars2 isempty2 cdata2 children2
+	if {[string equal $tag2 group]} {
+	    lappend groups $cdata2
+	}
+    }
+
+    if {$merge && [info exists bookmarks($connid,jid,$jid)]
+		&& $bookmarks($connid,hasgroups,$jid)} {
+	return 0
+    } else {
+	set bookmarks($connid,groups,$jid) $groups
+	set bookmarks($connid,hasgroups,$jid) 1
+	return 1
+    }
+}
+
+proc conferences::push_bookmarks_to_roster {connid} {
+    variable bookmarks
 
     foreach idx [array names bookmarks $connid,jid,*] {
 	set jid $bookmarks($idx)
@@ -158,7 +207,6 @@ proc conferences::process_bookmarks {connid res child} {
 			   bookmark ""
 	roster::override_category_and_subtype $connid $jid conference ""
     }
-    after idle [list [namespace current]::autojoin_groups $connid]
 }
 
 ###############################################################################
@@ -166,8 +214,7 @@ proc conferences::process_bookmarks {connid res child} {
 #   Store bookmarks
 #
 
-proc conferences::store_bookmarks {connid} {
-    variable NS
+proc conferences::serialize_bookmarks {connid} {
     variable bookmarks
 
     set bookmarklist {}
@@ -201,22 +248,31 @@ proc conferences::store_bookmarks {connid} {
 			       -subtags $groups]
     }
 
-    jlib::send_iq set \
-	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $NS(private)] \
-	     -subtags [list [jlib::wrapper:createtag storage \
-				 -vars [list xmlns $NS(bookmarks)] \
-				 -subtags $bookmarklist]]] \
-	-command [list [namespace current]::store_bookmarks_result $connid] \
-	-connection $connid
-    jlib::send_iq set \
-	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $NS(private)] \
-	     -subtags [list [jlib::wrapper:createtag storage \
-				 -vars [list xmlns $NS(tkabber:groups)] \
-				 -subtags $grouplist]]] \
-	-command [list [namespace current]::store_bookmarks_result $connid] \
-	-connection $connid
+    list [jlib::wrapper:createtag storage \
+	    -vars [list xmlns $::NS(bookmarks)] \
+	    -subtags $bookmarklist] \
+	[jlib::wrapper:createtag storage \
+	    -vars [list xmlns $::NS(tkabber:groups)] \
+	    -subtags $grouplist]
+}
+
+proc conferences::store_bookmarks {connid args} {
+    set command [list [namespace current]::store_bookmarks_result $connid]
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -command { set command $val }
+	    default {
+		return -code error "Bad option \"$opt\":\
+		    must be -command"
+	    }
+	}
+    }
+
+    foreach item [serialize_bookmarks $connid] {
+	private::store [list $item] \
+	    -command $command \
+	    -connection $connid
+    }
 }
 
 proc conferences::store_bookmarks_result {connid res child} {
@@ -226,6 +282,7 @@ proc conferences::store_bookmarks_result {connid res child} {
     if {[winfo exists .store_bookmarks_error]} {
 	return
     }
+
     MessageDlg .store_bookmarks_error -aspect 50000 -icon error \
 	-message [format [::msgcat::mc "Storing conferences failed: %s"] \
 			 [error_to_string $child]] \
@@ -397,7 +454,8 @@ proc conferences::add_conference {gw} {
 	}
     }
     if {![info exists connid]} {
-	set connid [jlib::route $jid]
+	# Disconnect while dialog is opened
+	return
     }
 
     if {[info exists bookmarks($connid,jid,$jid)]} {
@@ -453,6 +511,7 @@ proc conferences::add_bookmark {connid jid args} {
     set bookmarks($connid,password,$jid) $password
     set bookmarks($connid,autojoin,$jid) $autojoin
     set bookmarks($connid,groups,$jid) $groups
+    set bookmarks($connid,hasgroups,$jid) 1
 
     # TODO should we remove $jid from the roster if it is here?
     client:roster_push $connid $jid $name $groups bookmark ""
@@ -549,6 +608,7 @@ proc conferences::remove_bookmark {connid jid} {
     catch { unset bookmarks($connid,password,$jid) }
     catch { unset bookmarks($connid,autojoin,$jid) }
     catch { unset bookmarks($connid,groups,$jid) }
+    catch { unset bookmarks($connid,hasgroups,$jid) }
 
     store_bookmarks $connid
 
@@ -880,5 +940,4 @@ proc conferences::disco_node_menu_setup {m bw tnode data parentdata} {
 hook::add disco_node_menu_hook \
 	  [namespace current]::conferences::disco_node_menu_setup 50
 
-###############################################################################
-
+# vim:ts=8:sw=4:sts=4:noet
