@@ -1,4 +1,4 @@
-# $Id: taskbar.tcl 1043 2007-03-10 19:29:19Z sergei $
+# $Id: taskbar.tcl 1328 2007-12-16 14:08:35Z sergei $
 
 # MS Windows taskbar and tray icon support.
 # Requires Winico package
@@ -47,10 +47,33 @@ namespace eval taskbar {
 
 proc taskbar::set_current_theme {} {
     variable icon
+    variable s2p
+    global userstatus
 
-    if {[info exists icon]} return
+    set newicon [winico createfrom [pixmaps::get_filename docking/tkabber]]
 
-    set icon [winico createfrom [pixmaps::get_filename docking/tkabber]]
+    if {[info exists icon]} {
+	set oldicon $icon
+	set icon $newicon
+
+	# Change taskbar icon:
+	winico taskbar add $newicon
+	# TODO ideally there should be a call to [enable_disable]
+	# instead but the latter relies on the existence of some
+	# windows (i.e. it's called from finload_hook) while this
+	# proc is called for the first time earlier in the loading
+	# sequence...
+	configure .tray $userstatus
+
+	# Change icons of all mapped toplevels:
+	foreach w [wm stackorder .] {
+	    win_icon_setup $w
+	}
+
+	winico delete $oldicon
+    } else {
+	set icon $newicon
+    }
 }
 
 hook::add set_theme_hook [namespace current]::taskbar::set_current_theme
@@ -115,7 +138,11 @@ proc taskbar::destroy {m} {
 proc taskbar::callback {m event x y} {
     switch -- $event {
 	WM_LBUTTONUP {
-	    ifacetk::systray::toggle_state
+	    ifacetk::systray::restore
+	}
+
+	WM_MBUTTONUP {
+	    ifacetk::systray::withdraw
 	}
 
 	WM_RBUTTONUP {
@@ -139,29 +166,36 @@ proc taskbar::win_icons {} {
 
     trace variable ::curuserstatus w [namespace code update]
 
-    foreach tag [list Toplevel Chat JBrowser JDisco] {
-	bind $tag <Map> \
-	     [list [namespace code win_icon_setup] %W $tag]
-    }
+    bind all <Map> +[namespace code {
+	if {[string equal [winfo toplevel %W] %W]} { win_icon_setup %W }
+    }]
 }
 
 hook::add finload_hook [namespace current]::taskbar::win_icons
 
 ##########################################################################
 
-proc taskbar::win_icon_setup {w tag} {
+proc taskbar::win_icon_setup {w} {
     variable icon
     variable s2p
+    global userstatus
 
     if {[cequal $icon ""]} return
-    if {[winfo toplevel $w] != $w} return
 
-    switch -- $tag {
+    # Special case for the main window (which is also the roster
+    # window in windowed UI mode) -- it shows the current
+    # user status in its window icon:
+    if {[string equal $w .]} {
+	winico setwindow $w $icon small $s2p($userstatus)
+	winico setwindow $w $icon big $s2p(available32)
+	return
+    }
+
+    switch -- [winfo class $w] {
 	Chat {
 	    winico setwindow $w $icon small $s2p(group)
 	    winico setwindow $w $icon big $s2p(group32)
 	}
-	JBrowser -
 	JDisco {
 	    winico setwindow $w $icon small $s2p(browser)
 	    winico setwindow $w $icon big $s2p(browser32)
