@@ -1,4 +1,4 @@
-# $Id: roster.tcl 1087 2007-04-08 17:27:04Z sergei $
+# $Id: roster.tcl 1425 2008-05-18 07:29:04Z sergei $
 
 namespace eval roster {
     variable undef_group_name [::msgcat::mc "Undefined"]
@@ -194,12 +194,20 @@ proc roster::itemconfig {connid jid args} {
 	    -name     {set param name}
 	    -subsc    {set param subsc}
 	    -ask      {set param ask}
-	    -category {set param category}
-	    -subtype  {set param subtype}
+	    -category {
+		return [lindex [get_category_and_subtype $connid $jid] 0]
+	    }
+	    -subtype  {
+		return [lindex [get_category_and_subtype $connid $jid] 1]
+	    }
 	    -isuser   {
 		return [cequal [lindex [get_category_and_subtype $connid $jid] 0] "user"]
 	    }
-	    default   {return -code error "Illegal option"}
+	    default   {
+		return -code error "Bad option \"$attr\":\
+		    must be one of: -group, -name, -subsc, -ask,\
+		    -category, -subtype or -isuser"
+	    }
 	}
 	if {[info exists roster($param,$connid,$jid)]} {
 	    return $roster($param,$connid,$jid)
@@ -213,8 +221,14 @@ proc roster::itemconfig {connid jid args} {
 		-name     {set param name}
 		-subsc    {set param subsc}
 		-ask      {set param ask}
-		-category {set param category}
-		-subtype  {set param subtype}
+		-category {
+		    override_category $connid $jid $val
+		    continue
+		}
+		-subtype  {
+		    override_subtype $connid $jid $val
+		    continue
+		}
 		default   {return -code error "Illegal option"}
 	    }
 	    set roster($param,$connid,$jid) $val
@@ -222,6 +236,20 @@ proc roster::itemconfig {connid jid args} {
     }
 }
 
+# Returns true if $jid is allowed to receive our presence information,
+# false otherwise.
+proc roster::is_trusted {connid jid} {
+    set subsc [itemconfig $connid [find_jid $connid $jid] -subsc]
+
+    if {[node_and_server_from_jid $jid] == [jlib::connection_bare_jid $connid]} {
+	return 1
+    } elseif {$subsc == "both" || $subsc == "from"} {
+	return 1
+    } else {
+	return 0
+    }
+}
+
 proc roster::on_change_jid_presence {connid jid type x args} {
     variable roster
 
@@ -292,6 +320,36 @@ proc roster::override_category_and_subtype {connid jid category subtype} {
 	[list $category $subtype]
 }
 
+proc roster::override_category {connid jid category} {
+    variable roster
+
+    if {![info exists roster(overridden_category_and_subtype,$connid,$jid)]} {
+	lassign [get_category_and_subtype $connid $jid] category1 subtype
+	set roster(overridden_category_and_subtype,$connid,$jid) \
+	    [list $category $subtype]
+    } else {
+	set roster(overridden_category_and_subtype,$connid,$jid) \
+	    [list $category \
+		  [lindex \
+		       $roster(overridden_category_and_subtype,$connid,$jid) 1]]
+    }
+}
+
+proc roster::override_subtype {connid jid subtype} {
+    variable roster
+
+    if {![info exists roster(overridden_category_and_subtype,$connid,$jid)]} {
+	lassign [get_category_and_subtype $connid $jid] category subtype1
+	set roster(overridden_category_and_subtype,$connid,$jid) \
+	    [list $category $subtype]
+    } else {
+	set roster(overridden_category_and_subtype,$connid,$jid) \
+	    [list [lindex \
+		       $roster(overridden_category_and_subtype,$connid,$jid) 0] \
+		  $subtype]
+    }
+}
+
 proc roster::get_category_and_subtype {connid jid} {
     variable roster
 
@@ -401,7 +459,6 @@ proc roster::clean {} {
     array unset roster name,*
     array unset roster subsc,*
     array unset roster ask,*
-    array unset roster category,*
     array unset roster subtype,*
     array unset roster cached_category_and_subtype,*
     array unset roster overridden_category_and_subtype,*
@@ -416,7 +473,6 @@ proc roster::clean_connection {connid} {
     array unset roster name,$connid,*
     array unset roster subsc,$connid,*
     array unset roster ask,$connid,*
-    array unset roster category,$connid,*
     array unset roster subtype,$connid,*
     array unset roster cached_category_and_subtype,$connid,*
     array unset roster overridden_category_and_subtype,$connid,*
