@@ -2,13 +2,11 @@
 # the next line restarts using the correct interpreter \
 exec wish "$0" -name tkabber "$@"
 
-# $Id: tkabber.tcl 1108 2007-04-12 16:14:22Z sergei $
+# $Id: tkabber.tcl 1456 2008-06-08 13:33:26Z sergei $
 
 set interface tk
 namespace eval ifacetk {}
 
-wm withdraw .
-
 set aquaP 0
 catch {
     switch -- [tk windowingsystem] {
@@ -60,8 +58,6 @@ if {[info exists ::env(LC_MESSAGES)]} {
     ::msgcat::mclocale $::env(LC_MESSAGES)
 }
 
-package require BWidget
-
 tk appname tkabber
 
 set debug_lvls {}
@@ -104,7 +100,7 @@ proc get_snapshot {changelog} {
     return $snapshot
 }
 
-set tkabber_version "0.10.0"
+set tkabber_version "0.11.0"
 set toolkit_version "Tcl/Tk [info patchlevel]"
 
 proc rescmd {id res ls} {
@@ -137,7 +133,6 @@ hook::add quit_hook {
 } 100
 
 load_source default.tcl
-load_source ifacetk idefault.tcl
 
 hook::add postload_hook postload
 hook::add finload_hook finload
@@ -151,12 +146,14 @@ if {[file exists [file join $configdir config.tcl]]} {
     source [file join $configdir config.tcl]
 }
 
+package require BWidget
+
 if {![info exists load_default_xrdb] || $load_default_xrdb} {
-    option readfile [fullpath ifacetk default.xrdb] startupFile
+    option readfile [fullpath ifacetk default.xrdb] 21
 
     switch -- $tcl_platform(platform) {
 	unix {
-	    option readfile [fullpath ifacetk unix.xrdb] startupFile
+	    option readfile [fullpath ifacetk unix.xrdb] 21
 	}
     }
 }
@@ -182,11 +179,9 @@ unset pr f
 
 if {[catch { package require Tclx }]} {
     load_source Tclx.tcl
-} else {
-    random seed [expr {round(1000000000*rand())}]
 }
 
-package require jabberlib 0.10.0
+package require jabberlib 0.10.1
 load_source xmppmime.tcl
 
 foreach {opt val} $argv {
@@ -226,6 +221,7 @@ if {[catch { package require Img }]} {
 The IMG package is available at http://www.xs4all.nl/~nijtmans/img.html"
 }
 
+load_source ifacetk idefault.tcl
 load_source custom.tcl
 load_source utils.tcl
 load_source plugins.tcl
@@ -233,6 +229,7 @@ load_source pixmaps.tcl
 load_source balloon.tcl
 load_source presence.tcl
 load_source iq.tcl
+load_source disco.tcl
 load_source roster.tcl
 load_source itemedit.tcl
 load_source messages.tcl
@@ -240,21 +237,20 @@ load_source chats.tcl
 load_source joingrdialog.tcl
 load_source muc.tcl
 load_source login.tcl
-load_source browser.tcl
-load_source disco.tcl
 load_source userinfo.tcl
 load_source datagathering.tcl
 load_source negotiate.tcl
 load_source mclistbox mclistbox.tcl
 load_source search.tcl
 load_source register.tcl
-load_source jidlink.tcl
 load_source si.tcl
 load_source filetransfer.tcl
 load_source filters.tcl
 load_source privacy.tcl
 load_source gpgme.tcl
 load_source pubsub.tcl
+load_source pep.tcl
+load_source private.tcl
 load_source richtext.tcl
 
 load_source ifacetk bwidget_workarounds.tcl
@@ -264,6 +260,7 @@ plugins::load [file join plugins general]
 plugins::load [file join plugins roster]
 plugins::load [file join plugins search]
 plugins::load [file join plugins richtext]
+plugins::load [file join plugins pep]
 plugins::load [file join plugins $tcl_platform(platform)]
 if {[info exists env(TKABBER_SITE_PLUGINS)] && \
 	[file isdirectory $env(TKABBER_SITE_PLUGINS)]} {
