@@ -1,4 +1,8 @@
-# $Id: utils.tcl 949 2007-02-16 20:53:05Z sergei $
+# $Id: utils.tcl 1436 2008-05-20 05:48:52Z sergei $
+
+proc rand {num} {
+    return [expr round(rand()*$num)]
+}
 
 proc user_from_jid {jid} {
     set user $jid
@@ -87,9 +91,9 @@ proc jid_to_tag {jid} {
 	return $jidtag($jid)
     } else {
 	regsub -all {[^[:alnum:]]+} $jid {} prefix
-	set tag $prefix[random 1000000000]
+	set tag $prefix[rand 1000000000]
 	while {[info exists tagjid($tag)]} {
-	    set tag $prefix[random 1000000000]
+	    set tag $prefix[rand 1000000000]
 	}
 
 	set jidtag($jid) $tag
@@ -201,7 +205,6 @@ proc format_time {t} {
 }
 
 proc NonmodalMessageDlg {path args} {
-
     set icon "none"
     set title ""
     set message ""
@@ -511,6 +514,16 @@ proc lfilter {command list} {
     return $newlist
 }
 
+# Removes $nth element from the list contained in a
+# variable named $listVar in the caller's scope,
+# then returns the value of the removed element.
+proc lpop {listVar {nth 0}} {
+    upvar 1 $listVar L
+    set v [lindex $L $nth]
+    set L [lreplace $L $nth $nth]
+    set v
+}
+
 # Returns a fully-qualified name of the command that has invoked
 # the caller of this procedure.
 # To put is simple: if ::one::bar has invoked ::two::foo, the
@@ -530,4 +543,70 @@ proc caller {{level 1}} {
     }
 }
 
+##################################################################
+
+proc reverse_scroll {w} {
+    set command [$w cget -yscrollcommand]
+    $w configure -yscrollcommand [list store_scroll $w $command]
+    bind $w <Configure> {move_scroll %W}
+    bind $w <Destroy> {+clean_scroll %W}
+}
+
+proc store_scroll {w command lo hi} {
+    set ::lo($w) $lo
+    set ::hi($w) $hi
+    eval $command {$lo $hi}
+}
+
+proc move_scroll {w} {
+    if {![info exists ::lo($w)] || ![info exists ::hi($w)]} return
+    foreach {lo hi} [$w yview] break
+    if {$hi < 1.0} {
+	$w yview moveto [expr {$lo + ($::hi($w) - $::lo($w)) - ($hi - $lo)}]
+    } else {
+	$w see end
+    }
+}
+
+proc clean_scroll {w} {
+    catch {unset ::lo($w)}
+    catch {unset ::hi($w)}
+}
+
+##################################################################
+
+proc epath {} {
+    global EPathNum
+    if {![info exists EPathNum]} {
+	set EPathNum 0
+    } else {
+	incr EPathNum
+    }
+    return .errorpath$EPathNum
+}
+
+##################################################################
+
+proc get_conf {w option} {
+    return [lindex [$w configure $option] 4]
+}
+
+##################################################################
+
+proc render_url {path url title args} {
+    set t [eval [list text $path \
+	       -cursor left_ptr \
+	       -height 1 \
+	       -width 10 \
+	       -bd 0 \
+	       -highlightthickness 0 \
+	       -takefocus 0 \
+	       -wrap none] $args]
+    ::richtext::config $t -using url
+    ::plugins::urls::render_url $t text $url {} -title $title
+    $t delete {end - 1 char}
+    $t configure -state disabled
+    return $t
+}
+
 # vim:ts=8:sw=4:sts=4:noet
